#!/bin/sh
# Run wish from the user's PATH \
exec wish -f "$0" ${1+"$@"}
#
#
#
#	Zmapper.tcl
#
#
#  -----------------------------------------------------------------------
#  -----------------------------------------------------------------------
#  Copyright 2009  Alfred Steffens Jr.
#  -----------------------------------------------------------------------
#  -----------------------------------------------------------------------
#    Copying Permission:
#
#    Zmapper is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation.
#
#    Zmapper is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Zmapper (see file called "COPYING"); if not, write to the Free
#    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
#    02111-1307  USA
#  ------------------------------------------------------------------------

tk appname "Zmapper - - -  complex plane mapping tool  - - -  ver. 1.1.1"
set configfile_name "zmapper.cfg"

#	--------------------------------------------------------------------
#	Change Log
#	----------
#   ver. 1,  revision 1, build 1
#   Initial release.
#	--------------------------------------------------------------------

#   -------------------------------------------------------------------------
#	Canvas width and Height
#
#	(even though it is recalculated below with
#	wd_Client and ht_Client - this allows us
#	to easily set the canvas dimensions)
#
    set Cwidth 800
    set Cheight 600
    set MB_ht 50
    set DB_ht 20
    set use_canvas_scrollbars 1
    set canvas_magnification 10
#
#
#	Program window (pretend as if BaseWindow is the top)
#
set wd_BaseWindow $Cwidth
set ht_BaseWindow [expr $Cheight + $MB_ht + $DB_ht + $DB_ht]
#
#	interior frame just for the menu bar
#
set wd_MenuBar $wd_BaseWindow
set ht_MenuBar $MB_ht
#
#	interior frame just for the display bar
#
set wd_DisplayBar $wd_BaseWindow
set ht_DisplayBar $DB_ht
#
#	interior frame just for the display bar
#
set wd_DisplayBar2 $wd_BaseWindow
set ht_DisplayBar2 $DB_ht
#
#   Client (canvas)
#
set wd_Client $wd_BaseWindow
set ht_Client [expr $ht_BaseWindow - $ht_MenuBar - $ht_DisplayBar - $ht_DisplayBar2]

set df_file_name "none"
set df_file_folder [pwd]
set df_file_fullpath [file join $df_file_folder $df_file_name]
set df_canvas .f_client.c1
set df_draw_color "lightblue"
set df_map_color "red"

set curr_file_fullpath $df_file_fullpath
set curr_file_name $df_file_name
set curr_file_folder $df_file_folder
set curr_canvas $df_canvas
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   More constants
#
set xdivscale 0.01
set ydivscale 0.01
set tic_ht 4
set tic_wd 4
set xlabel_above 0
set ylabel_right 0
set xlabel_xshift 0
set xlabel_yshift -6
set ylabel_xshift -6
set ylabel_yshift 0
#	---------------------------------------------------------------------

#	---------------------------------------------------------------------
#
#   Flags
#
#	figure modes:
#	1  =  freehand
#	2  =  lines
#	3  =  functions
#
set figure_mode 1
set figmode_label_1 "free hand"
set figmode_label_2 "lines"
set figmode_label_3 "ortho lines"
set figmode_label_4 "functions"
set figmode_label_5 "data"
#
#	mapping modes:
#	1	inversion in the unit circle
#
set mapping_mode 1
set mapmode_label_1 "inv in unit circle"
set mapmode_label_2 "ln(z)"
set mapmode_label_3 "exp(z)"
#
#	flag that turns on the coordinates monitor
#
set coords_watch 1
#
#   use an x axis
#
set xaxis_mode 1
#
#   use a y axis
#
set yaxis_mode 1
#
#   use x tics on the axis
#
set xtics_mode 1
#
#   use y tics on the axis
#
set ytics_mode 1
#
#   use x labels on the axis
#
set xlabels_mode 1
#
#   use y labels on the axis
#
set ylabels_mode 1
#
#   Point style:
#       1 = dots
#       2 = ovals
#       3 = lines
#
set pen_style 1
set pnstyle_1 1
set pnstyle_2 0
set pnstyle_3 0
#
#
#
set max_ortho_lines 3
set aspect_ortho_lines 0.5
set Curr_Map_File ""
#	---------------------------------------------------------------------

#	---------------------------------------------------------------------
#   Internal Globals    (keep out of config file)
set xrange ""
set yrange ""
set tp_Mu 0
set tp_Bu 0
set tp_Mv 0
set tp_Bv 0
set tp_Mx 0
set tp_Bx 0
set tp_My 0
set tp_By 0
set id_xaxis -1
set id_yaxis -1
set id_unit_circle -1
set B1_down 0
set xdiv 1
set ydiv 1
#
set save_map_file_flag 1
set curr_draw_color $df_draw_color
set curr_map_color $df_map_color
set max_mapping_mode 0
set Last_Predefined_Map_Mode 3
set PI 3.1415927
#	---------------------------------------------------------------------

#   -------------------------------------------------------------------------
#   Scale and Offset:
#   This is what the user changes to zoom or shift the screen.
#
#	Scaling (zoom size) scaling in units/pixel
#	Larger zoom = smaller Scaling
#   This scale factor is the number of mathemtical units per pixel.
#
set Scaling 0.01
set Scaling_last $Scaling
set coord_units "dimensionless units"
set coord_scaling "$Scaling units/pixel"
#
#	unit coordinates corresponding to the center of the screen
#	(the user will choose this for shifting the coordinates)
#	default is the origin at screen center
#   In other words, if you position the mouse at the center of the screen,
#   what x,y (mathematical units) coordinates will be shown.
#
set xcenter 0
set ycenter 0
set xcenter_last $xcenter
set ycenter_last $ycenter
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------
#   Some basic startup procedures, needed below
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------

#   -------------------------------------------------------------------------
#   Procedure -- Error_Message
#
#   Description
#
#       Error message popup window.
#
#   Arguments
#
#       msg -- a text string containing the error message.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Error_Message { msg } {
   tk_messageBox -type ok -title Error -message\
               $msg -icon error
}
#  proc -- Error_Message
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Warning_Message
#
#   Description
#
#       Warning message popup window.
#
#   Arguments
#
#       msg -- a text string containing the  message.
#
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Warning_Message { msg } {
   tk_messageBox -type ok -title Warning -message\
               $msg -icon warning
}
#  proc -- Warning_Message
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Info_Message
#
#   Description
#
#       Information message popup window.
#
#   Arguments
#
#       msg -- a text string containing the message.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Info_Message { msg } {
   tk_messageBox -type ok -title Information -message\
               $msg -icon info
}
#  proc -- Info_Message
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Pixel_Mu
#
#   Description
#
#       Convert mathematical coordinate to pixel coordinate.
#       THIS CALCULATES THE MULTIPLIER, Mu.
#       px       = Mu * x    + Bu
#       using endpoints
#       0        = Mu * xmin + Bu
#       px_width = Mu * xmax + Bu
#       Mu = px_width / (xmax-xmin)
#       Bu = px_width * xmin / (xmin-xmax)
#
#   Arguments
#
#       xmin
#
#           The mathematical x coordinate at the far left of the screen.
#
#       xmax
#
#           The mathematical x coordinate at the far right of the screen.
#
#       pxwid
#
#           The pixel width of the screen (px_width).
#
#   Returns
#
#       Returns the multiplier, Mu, for calculating the pixel x coordinate,
#       px.
#
#   -------------------------------------------------------------------------
proc Pixel_Mu { xmin xmax pxwid } {
    #
    #   Denominator (determinent of transformation)
    #
    set det [expr $xmin - $xmax]
    if { $det == 0 } {
        # Error condition
    }

    set Mu [expr $pxwid * (-1) / $det]

    return $Mu
}
#   procedure -- Pixel_Mu
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Pixel_Bu
#
#   Description
#
#       Convert mathematical coordinate to pixel coordinate.
#       THIS CALCULATES THE OFFSET, Bu.
#       px       = Mu * x    + Bu
#       using endpoints
#       0        = Mu * xmin + Bu
#       px_width = Mu * xmax + Bu
#       Mu = px_width / (xmax-xmin)
#       Bu = px_width * xmin / (xmin-xmax)
#
#   Arguments
#
#       xmin
#
#           The mathematical x coordinate at the far left of the screen.
#
#       xmax
#
#           The mathematical x coordinate at the far right of the screen.
#
#       pxwid
#
#           The pixel width of the screen (px_width).
#
#   Returns
#
#       Returns the offset, Bu, for calculating the pixel x coordinate,
#       px.
#
#   -------------------------------------------------------------------------
proc Pixel_Bu { xmin xmax pxwid } {
    #
    #   Denominator (determinent of transformation)
    #
    set det [expr $xmin - $xmax]
    if { $det == 0 } {
        # Error condition
    }

    set Bu [expr $pxwid * $xmin / $det]

    return $Bu
}
#   procedure -- Pixel_Bu
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Pixel_x
#
#   Description
#
#       Convert mathematical coordinate to pixel coordinate.
#       px       = Mu * x    + Bu
#
#   Arguments
#
#       x
#
#           The mathematical x coordinate.
#
#       Mu
#
#           The multiplier, calculated with Pixel_Mu.
#
#       Bu
#
#           The offset, calculated with Pixel_Bu.
#
#
#   Returns
#
#       Returns the pixel coordinate, px, corresponding to the mathematical
#       coordinate x.
#
#   -------------------------------------------------------------------------
proc Pixel_x { x Mu Bu } {

    set px [expr $Mu * $x + $Bu]

    return $px
}
#   procedure -- Pixel_x
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Pixel_Mv
#
#   Description
#
#       Convert mathematical coordinate to pixel coordinate.
#       THIS CALCULATES THE MULTIPLIER, Mv.
#       py       = Mv * y    + Bv
#       using endpoints
#       0         = Mv * ymax + Bv
#       py_height = Mv * ymin + Bv
#       Mv = -py_height / (ymax-ymin)
#       Bv = py_height * ymax / (ymax-ymin)
#
#   Arguments
#
#       ymin
#
#           The mathematical y coordinate at the far bottom of the screen.
#
#       ymax
#
#           The mathematical y coordinate at the far top of the screen.
#
#       pyht
#
#           The pixel height of the screen (py_height).
#
#   Returns
#
#       Returns the multiplier, Mv, for calculating the pixel y coordinate,
#       py.
#
#   -------------------------------------------------------------------------
proc Pixel_Mv { ymin ymax pyht } {
    #
    #   Denominator (determinent of transformation)
    #
    set det [expr $ymax - $ymin]
    if { $det == 0 } {
        # Error condition
    }

    set Mv [expr $pyht * (-1) / $det]

    return $Mv
}
#   procedure -- Pixel_Mv
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Pixel_Bv
#
#   Description
#
#       Convert mathematical coordinate to pixel coordinate.
#       THIS CALCULATES THE OFFSET, Bv.
#       py       = Mv * y    + Bv
#       using endpoints
#       0         = Mv * ymax + Bv
#       py_height = Mv * ymin + Bv
#       Mv = -py_height / (ymax-ymin)
#       Bv = py_height * ymax / (ymax-ymin)
#
#   Arguments
#
#       ymin
#
#           The mathematical y coordinate at the far bottom of the screen.
#
#       ymax
#
#           The mathematical y coordinate at the far top of the screen.
#
#       pyht
#
#           The pixel height of the screen (py_height).
#
#   Returns
#
#       Returns the offset, Bv, for calculating the pixel y coordinate,
#       py.
#
#   -------------------------------------------------------------------------
proc Pixel_Bv { ymin ymax pyht } {
    #
    #   Denominator (determinent of transformation)
    #
    set det [expr $ymax - $ymin]
    if { $det == 0 } {
        # Error condition
    }

    set Bv [expr $pyht * $ymax / $det]

    return $Bv
}
#   procedure -- Pixel_Bv
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Pixel_y
#
#   Description
#
#       Convert mathematical coordinate to pixel coordinate.
#       py       = Mv * y    + Bv
#
#   Arguments
#
#       y
#
#           The mathematical y coordinate.
#
#       Mv
#
#           The multiplier, calculated with Pixel_Mv.
#
#       Bv
#
#           The offset, calculated with Pixel_Bv.
#
#
#   Returns
#
#       Returns the pixel coordinate, py, corresponding to the mathematical
#       coordinate y.
#
#   -------------------------------------------------------------------------
proc Pixel_y { y Mv Bv } {

    set py [expr $Mv * $y + $Bv]

    return $py
}
#   procedure -- Pixel_y
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Coord_Mx
#
#   Description
#
#       Convert pixel coordinate mathematical coordinate.
#       THIS CALCULATES THE MULTIPLIER, Mx.
#       x       = Mx * u    + Bx
#       using endpoints
#       xmin = Mx * umin + Bx
#       xmax = Mx * umax + Bx
#
#       xmin = 0  +  Bx
#       Bx = xmin
#       xmax = Mx * px_width + Bx
#       Mx = (xmax-xmin)/px_width
#
#   Arguments
#
#       xmin
#
#           The mathematical x coordinate at the far left of the screen.
#
#       xmax
#
#           The mathematical x coordinate at the far right of the screen.
#
#       pxwid
#
#           The pixel width of the screen (px_width).
#
#   Returns
#
#       Returns the multiplier, Mx, for calculating the math coordinate, x.
#
#   -------------------------------------------------------------------------
proc Coord_Mx { xmin xmax pxwid } {
    #
    #   Denominator (determinent of transformation)
    #
    set det [expr $xmax - $xmin]
    if { $det == 0 } {
        # Error condition
    }

    set Mx [expr $det / $pxwid]

    return $Mx
}
#   procedure -- Coord_Mx
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Coord_Bx
#
#   Description
#
#       Convert pixel coordinate mathematical coordinate.
#       THIS CALCULATES THE OFFSET, Bx.
#       x       = Mx * u    + Bx
#       using endpoints
#       xmin = Mx * umin + Bx
#       xmax = Mx * umax + Bx
#
#       xmin = 0  +  Bx
#       Bx = xmin
#       xmax = Mx * px_width + Bx
#       Mx = (xmax-xmin)/px_width
#
#   Arguments
#
#       xmin
#
#           The mathematical x coordinate at the far left of the screen.
#
#       xmax
#
#           The mathematical x coordinate at the far right of the screen.
#
#       pxwid
#
#           The pixel width of the screen (px_width).
#
#   Returns
#
#       Returns the offset, Bx, for calculating the math coordinate, x.
#
#   -------------------------------------------------------------------------
proc Coord_Bx { xmin xmax pxwid } {
    #
    #   The xmax and pxwid arguments are merely for making the argument
    #   list consistent with the Mx calculation.
    #

    return $xmin
}
#   procedure -- Coord_Bx
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Coord_x
#
#   Description
#
#       Convert pixel x coordinate into the mathematical x coordinate.
#       x       = Mx * px    + Bx
#
#   Arguments
#
#       px
#
#           The pixel x coordinate.
#
#       Mx
#
#           The multiplier, calculated with Coord_Mx
#
#       Bx
#
#           The offset, calculated with Coord_Bx.
#
#
#   Returns
#
#       Returns the mathematical coordinate, x, corresponding to the pixel
#       coordinate px.
#
#   -------------------------------------------------------------------------
proc Coord_x { px Mx Bx } {

    set x [expr $Mx * $px + $Bx]

    return $x
}
#   procedure -- Coord_x
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Coord_My
#
#   Description
#
#       Convert pixel coordinate mathematical coordinate.
#       THIS CALCULATES THE MULTIPLIER, My.
#       y       = My * v    + By
#       using endpoints
#       ymin = My * vmax + By
#       ymax = My * vmin + By
#
#       ymax = 0 + By
#       By = ymax
#       ymin = My * py_height + By
#       My = (ymin-ymax) / py_height
#
#   Arguments
#
#       ymin
#
#           The mathematical y coordinate at the far bottom of the screen.
#
#       ymax
#
#           The mathematical y coordinate at the far top of the screen.
#
#       pyht
#
#           The pixel height of the screen (py_height).
#
#   Returns
#
#       Returns the multiplier, My, for calculating the math coordinate, y.
#
#   -------------------------------------------------------------------------
proc Coord_My { ymin ymax pyht } {
    #
    #   Denominator (determinent of transformation)
    #
    set det [expr $ymin - $ymax]
    if { $det == 0 } {
        # Error condition
    }

    set My [expr $det / $pyht]

    return $My
}
#   procedure -- Coord_My
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Coord_By
#
#   Description
#
#       Convert pixel coordinate mathematical coordinate.
#       THIS CALCULATES THE OFFSET, By.
#       y       = My * v    + By
#       using endpoints
#       ymin = My * vmax + By
#       ymax = My * vmin + By
#
#       ymax = 0 + By
#       By = ymax
#       ymin = My * py_height + By
#       My = (ymin-ymax) / py_height
#
#   Arguments
#
#       ymin
#
#           The mathematical y coordinate at the far bottom of the screen.
#
#       ymax
#
#           The mathematical y coordinate at the far top of the screen.
#
#       pyht
#
#           The pixel height of the screen (py_height).
#
#   Returns
#
#       Returns the offset, By, for calculating the math coordinate, y.
#
#   -------------------------------------------------------------------------
proc Coord_By { ymin ymax pyht } {
    #
    #   The arguments ymin and pyht are there to make the argument list
    #   consistent with the My procedure.
    #

    return $ymax
}
#   procedure -- Coord_By
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Coord_y
#
#   Description
#
#       Convert pixel y coordinate into the mathematical y coordinate.
#       y       = My * py    + By
#
#   Arguments
#
#       py
#
#           The pixel y coordinate.
#
#       My
#
#           The multiplier, calculated with Coord_My
#
#       By
#
#           The offset, calculated with Coord_By.
#
#
#   Returns
#
#       Returns the mathematical coordinate, y, corresponding to the pixel
#       coordinate py.
#
#   -------------------------------------------------------------------------
proc Coord_y { py My By } {

    set y [expr $My * $py + $By]

    return $y
}
#   procedure -- Coord_y
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- GetValue
#
#   Description
#
#       A generic entry dialog--taken from Brent Welch's Tcl/Tk book.
#
#
#   Arguments
#
#       mprompt
#
#           The argument "mprompt" is a text string to be displayed when
#           the dialog is invoked.
#
#   Returns
#
#       Returns the string that was entered, or an empty string if the
#       cancel button was clicked.
#
#   -------------------------------------------------------------------------
proc GetValue { mprompt } {
global prompt

    set f [toplevel .prompt -borderwidth 10] 
    message $f.msg -text $mprompt
    entry $f.entry -textvariable prompt(result)
    set b [frame $f.buttons -bd 10]
    pack $f.msg $f.entry $f.buttons -side top -fill x

    bind $f.entry <Return> {set prompt(ok) 1}
    bind $f.entry <Control-c> {set prompt(ok) 0}
    button $b.ok -text OK -command {set prompt(ok) 1}
    button $b.cancel -text Cancel -command {set prompt(ok) 0}
    pack $b.ok $b.cancel -side left -fill x

    focus $f.entry
    grab $f
    tkwait variable prompt(ok)
    grab release $f
    destroy $f
    if { $prompt(ok) } {
        return $prompt(result)
    } else {
        return {}
    }
}
#   proc -- GetValue
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Abs
#
#   Description
#
#       Calculate the absolute value of a number.
#
#   Arguments
#
#       val -- the number.
#
#   Returns
#
#       Returns the absolute value.
#
#   -------------------------------------------------------------------------
proc Abs { val } {
    return [expr abs($val)]
}
#   proc -- Abs
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Min
#
#   Description
#
#       Calculate the minimum value of two numbers.
#
#   Arguments
#
#       val1 -- a number.
#
#       val2 -- a number.
#
#   Returns
#
#       Returns the minimum value.
#
#   -------------------------------------------------------------------------
proc Min { val1 val2 } {

    if { $val1 < $val2 } {
        return $val1
    } else {
        return $val2
    }
}
#   proc -- Min
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Max
#
#   Description
#
#       Calculate the maximum value of two numbers.
#
#   Arguments
#
#       val1 -- a number.
#
#       val2 -- a number.
#
#   Returns
#
#       Returns the maximum value.
#
#   -------------------------------------------------------------------------
proc Max { val1 val2 } {

    if { $val1 > $val2 } {
        return $val1
    } else {
        return $val2
    }
}
#   proc -- Max
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Pos_Integer
#
#   Description
#
#       Calculates the absolute value and rounds to an integer.
#
#   Arguments
#
#       The number.
#
#   Returns
#
#       Returns the new number.
#
#   -------------------------------------------------------------------------
proc Pos_Integer { val } {
    return [expr round([expr abs($val)])]
}
#   proc -- Pos_Integer
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Label_Mapmode
#
#   Description
#
#       Gets the display label associated with a particluar mapping mode.
#
#   Arguments
#
#       val -- the integer value of the mapping mode.
#
#   Returns
#
#       Returns a string that will be displayed on the GUI showing the
#       current mapping mode.
#
#   -------------------------------------------------------------------------
proc Label_Mapmode { val } {
global mapmode_label_1
global mapmode_label_2
global mapmode_label_3
global Last_Predefined_Map_Mode max_mapping_mode
global mapname_list

    set ival [Pos_Integer $val]

    if { $ival == 0 } {
        return "map mode: none"
    } elseif { $ival == 1 } {
        return "map mode: $mapmode_label_1"
    } elseif { $ival == 2 } {
        return "map mode: $mapmode_label_2"
    } elseif { $ival == 3 } {
        return "map mode: $mapmode_label_3"
    } elseif { $ival > $Last_Predefined_Map_Mode } {

        if { $ival <= $max_mapping_mode } {

            set listnum [expr $ival - 1]
            set name [lindex $mapname_list $listnum]
            return "map mode: $name"

        } else {
            return "map mode ($ival): undefined"
        }

    } else {
        return "map mode ($ival): undefined"
    }
}
#   proc -- Label_Mapmode
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Label_Figmode
#
#   Description
#
#       Gets the display label associated with a particluar figure mode.
#
#   Arguments
#
#       val -- the integer value of the figure mode.
#
#   Returns
#
#       Returns a string that will be displayed on the GUI showing the
#       current figure mode.
#
#   -------------------------------------------------------------------------
proc Label_Figmode { val } {
global figmode_label_1 figmode_label_2
global figmode_label_3 figmode_label_4
global figmode_label_5

    set ival [Pos_Integer $val]

    if { $ival == 0 } {
        return "tool: none"
    } elseif { $ival == 1 } {
        return "tool: $figmode_label_1"
    } elseif { $ival == 2 } {
        return "tool: $figmode_label_2"
    } elseif { $ival == 3 } {
        return "tool: $figmode_label_3"
    } elseif { $ival == 4 } {
        return "tool: $figmode_label_4"
    } elseif { $ival == 5 } {
        return "tool: $figmode_label_5"
    } else {
        return "tool ($ival): undefined"
    }
}
#   proc -- Label_Figmode
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#  --------------------------------------------------------------------------
#   Procedure -- Set_PenStyle
#
#   Description
#
#       Recalculates the pen style inegers that are assigned to the pen
#       style menu so that enabling one will disable the others.
#
#   Arguments
#
#       penstyle -- the integer value of the pen style.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Set_PenStyle { penstyle } {
global pen_style
global pnstyle_1 pnstyle_2 pnstyle_3

    set pen_style $penstyle

    if { $pen_style == 1 } {
        set pnstyle_1 1
        set pnstyle_2 0
    } elseif { $pen_style == 2 } {
        set pnstyle_1 0
        set pnstyle_2 1
    } elseif { $pen_style == 3 } {
        set pnstyle_1 0
        set pnstyle_2 0
        set pnstyle_3 1
    }
}
#   proc -- Set_PenStyle
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#  --------------------------------------------------------------------------
#   Procedure -- ConfigFile
#
#   Description
#
#       Reads the config file from the current directory and sets many
#       of the operating mode variables to the previous values from the
#       last session (assuming that the user stopped the program with
#       Exit, not Quit).  When the user terminates the program with the
#       Exit menu item the current configuration will be saved in the
#       current directory.  If the user terminates with either the Quit
#       menu of the system close button the configuration will not be
#       saved.
#
#   Arguments
#
#       rwflag  -- 0 means read, 1 means write
#
#       filename -- the name of the config file
#
#   Returns
#
#       Void (the result is that global values will be set or saved).
#
#  --------------------------------------------------------------------------
proc ConfigFile { rwflag filename } {
#
#	rwflag == 0		means read
#	rwflag == 1		means write
#
#
global configfile_name
global Scaling coord_scaling
global xcenter ycenter
global xdivscale ydivscale
global tic_ht tic_wd
global xlabel_above ylabel_right
global xlabel_xshift xlabel_yshift
global ylabel_xshift ylabel_yshift
global mapping_mode figure_mode
global pen_style
global xaxis_mode yaxis_mode
global xtics_mode ytics_mode xlabels_mode ylabels_mode
global use_canvas_scrollbars canvas_magnification
global max_ortho_lines aspect_ortho_lines
global Curr_Map_File save_map_file_flag


    if { $rwflag eq 0 } {

        #
        #   catch a non-existent file
        #
        if { [string length $filename] == 0 || \
             [file exists $filename] == 0 } {
            return
        }

        #
        #	read
        #
        set fid [open $filename "r"]
        if { $fid >= 0 } {

            #
            #	loop through all lines in the file
            #
            while { [eof $fid] ne 1 } {
            set fline [gets $fid]

                set isComment [regexp {\#(.*)} $fline]
                if { $isComment == 1 } {
                    continue
                }
                
                #   Scaling
                if { [regexp {Scaling ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set Scaling [Abs $dummysub]
                    set coord_scaling "$Scaling units/pixel"
                }

                #   xcenter
                if { [regexp {xcenter ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xcenter [expr $dummysub * 1]
                }

                #   ycenter
                if { [regexp {ycenter ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ycenter [expr $dummysub * 1]
                }

                #   mapping_mode
                if { [regexp {mapping_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set mapping_mode [Pos_Integer $dummysub]
                }

                #   figure_mode
                if { [regexp {figure_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set figure_mode [Pos_Integer $dummysub]
                }

                #   pen_style
                if { [regexp {pen_style ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    Set_PenStyle [Pos_Integer $dummysub]
                }

                #   xaxis_mode
                if { [regexp {xaxis_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xaxis_mode [Pos_Integer $dummysub]
                }

                #   yaxis_mode
                if { [regexp {yaxis_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set yaxis_mode [Pos_Integer $dummysub]
                }

                #   xtics_mode
                if { [regexp {xtics_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xtics_mode [Pos_Integer $dummysub]
                }

                #   ytics_mode
                if { [regexp {ytics_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ytics_mode [Pos_Integer $dummysub]
                }

                #   xlabels_mode
                if { [regexp {xlabels_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xlabels_mode [Pos_Integer $dummysub]
                }

                #   ylabel_mode
                if { [regexp {ylabels_mode ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ylabels_mode [Pos_Integer $dummysub]
                }


                #
                #   Settable only in the config file
                #

                #   xdivscale
                if { [regexp {xdivscale ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xdivscale [Abs $dummysub]
                }

                #   ydivscale
                if { [regexp {ydivscale ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ydivscale [Abs $dummysub]
                }

                #   tic_wd
                if { [regexp {tic_wd ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set tic_wd [Pos_Integer $dummysub]
                }

                #   tic_ht
                if { [regexp {tic_ht ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set tic_ht [Pos_Integer $dummysub]
                }

                #   xlabel_above
                if { [regexp {xlabel_above ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xlabel_above [Pos_Integer $dummysub]
                }

                #   ylabel_right
                if { [regexp {ylabel_right ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ylabel_right [Pos_Integer $dummysub]
                }

                #   xlabel_xshift
                if { [regexp {xlabel_xshift ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xlabel_xshift [expr round($dummysub)]
                }

                #   xlabel_yshift
                if { [regexp {xlabel_yshift ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set xlabel_yshift [expr round($dummysub)]
                }

                #   ylabel_xshift
                if { [regexp {ylabel_xshift ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ylabel_xshift [expr round($dummysub)]
                }

                #   ylabel_yshift
                if { [regexp {ylabel_yshift ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set ylabel_yshift [expr round($dummysub)]
                }

                #   use_canvas_scrollbars
                if { [regexp {use_canvas_scrollbars ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set use_canvas_scrollbars [Pos_Integer $dummysub]
                }

                #   canvas_magnification
                if { [regexp {canvas_magnification ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set canvas_magnification [expr abs($dummysub)]
                }

                #   max_ortho_lines
                if { [regexp {max_ortho_lines ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set max_ortho_lines [Pos_Integer $dummysub]
                    if { $max_ortho_lines < 1 } {
                        set max_ortho_lines 1
                    }
                }

                #   aspect_ortho_lines
                if { [regexp {aspect_ortho_lines ([\+\-0-9.]*[0-9]+)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set aspect_ortho_lines [Abs $dummysub]
                    if { $aspect_ortho_lines <= 0 } {
                        set aspect_ortho_lines 0.1
                    } elseif { $aspect_ortho_lines > 1 } {
                        set aspect_ortho_lines 1
                    }
                }

                #   load a map file
                if { [regexp {map_file ([a-zA-Z\/\\]+[a-zA-Z0-9_\-.\/\\]*)} $fline dummy1 dummysub]\
                       eq 1 } {
                    set Curr_Map_File $dummysub
                }

            }
        }
    } else {
        #
        #	write
        #
        if { [string length $filename] == 0 } {
            Error_Message "bad config file name ($filename)"
            return
        }

        set fid [open $filename "w"]
        if { $fid >= 0 } {

            puts $fid "#"
            puts $fid "#   $configfile_name"
            puts $fid "#"

            #	Scaling
            puts $fid "#"
            puts $fid "#   The Scaling parameter sets the magnification"
            puts $fid "#   This is units per pixel (default 0.01)"
            puts $fid "#"
            puts $fid "Scaling $Scaling"

            #	xcenter
            puts $fid "#"
            puts $fid "#   The xcenter parameter sets the shift in x direction"
            puts $fid "#   This will be the x coordinate at screen center"
            puts $fid "#"
            puts $fid "xcenter $xcenter"

            #	ycenter
            puts $fid "#"
            puts $fid "#   The ycenter parameter sets the shift in y direction"
            puts $fid "#   This will be the y coordinate at screen center"
            puts $fid "#"
            puts $fid "ycenter $ycenter"

            #	mapping_mode
            puts $fid "#"
            puts $fid "#   The mapping_mode parameter determines the kind of"
            puts $fid "#   conformal mapping to be used"
            puts $fid "#    0: none"
            puts $fid "#    1: inversion in a unit circle"
            puts $fid "#    2: ln(z)"
            puts $fid "#    3: exp(z)"
            puts $fid "#"
            puts $fid "mapping_mode $mapping_mode"

            #	figure_mode
            puts $fid "#"
            puts $fid "#   The figure_mode parameter determines the kind of"
            puts $fid "#   drawing technique used:"
            puts $fid "#    1: free hand"
            puts $fid "#    2: lines (applied with mouse)"
            puts $fid "#    3: ortho lines (applied with mouse)"
            puts $fid "#    4: functions"
            puts $fid "#    5: data"
            puts $fid "#"
            puts $fid "#   This parameter is set with the Tools menu"
            puts $fid "figure_mode $figure_mode"

            #	xaxis_mode
            puts $fid "#"
            puts $fid "#   The xaxis_mode turns the x axis view on or off"
            puts $fid "#   This parameter is set with the View menu"
            puts $fid "#"
            puts $fid "xaxis_mode $xaxis_mode"

            #	yaxis_mode
            puts $fid "#"
            puts $fid "#   The yaxis_mode turns the y axis view on or off"
            puts $fid "#   This parameter is set with the View menu"
            puts $fid "#"
            puts $fid "yaxis_mode $yaxis_mode"

            #	xtics_mode
            puts $fid "#"
            puts $fid "#   The xtics_mode turns the x axis tics on or off"
            puts $fid "#   This parameter is set with the View menu"
            puts $fid "#"
            puts $fid "xtics_mode $xtics_mode"

            #	ytics_mode
            puts $fid "#"
            puts $fid "#   The ytics_mode turns the y axis tics on or off"
            puts $fid "#   This parameter is set with the View menu"
            puts $fid "#"
            puts $fid "ytics_mode $ytics_mode"

            #	xlabels_mode
            puts $fid "#"
            puts $fid "#   The xlabels_mode turns the x axis labels on or off"
            puts $fid "#   This parameter is set with the View menu"
            puts $fid "#"
            puts $fid "xlabels_mode $xlabels_mode"

            #	ylabels_mode
            puts $fid "#"
            puts $fid "#   The ylabels_mode turns the y axis labels on or off"
            puts $fid "#   This parameter is set with the View menu"
            puts $fid "#"
            puts $fid "ylabels_mode $ylabels_mode"

            #	pen_style
            puts $fid "#"
            puts $fid "#   The pen_style parameter determines the kind of"
            puts $fid "#   pen used:"
            puts $fid "#    1: dots"
            puts $fid "#    2: ovals"
            puts $fid "#    3: lines"
            puts $fid "#"
            puts $fid "#   This parameter is set with the Options menu"
            puts $fid "pen_style $pen_style"

            puts $fid "#"
            puts $fid "#    -------------------------------------------------"
            puts $fid "#    The following parameters are settable only in the"
            puts $fid "#    config file"
            puts $fid "#    -------------------------------------------------"
            puts $fid "#"

            #	xdivscale
            puts $fid "#"
            puts $fid "#   The xdivscale parameter determines the distance"
            puts $fid "#   between tics on the x axis as a function of the"
            puts $fid "#   Scaling parameter (default 0.01).  The distance"
            puts $fid "#   between x axis tics in mathematical units will"
            puts $fid "#   be the Scaling divided by xdivscale"
            puts $fid "#"
            puts $fid "xdivscale $xdivscale"

            #	ydivscale
            puts $fid "#"
            puts $fid "#   The ydivscale parameter determines the distance"
            puts $fid "#   between tics on the y axis as a function of the"
            puts $fid "#   Scaling parameter (default 0.01).  The distance"
            puts $fid "#   between y axis tics in mathematical units will"
            puts $fid "#   be the Scaling divided by ydivscale"
            puts $fid "#"
            puts $fid "ydivscale $ydivscale"

            #	tic_wd
            puts $fid "#"
            puts $fid "#   The tic_wd parameter determines the width of a"
            puts $fid "#   tic mark on the y axis, in pixels"
            puts $fid "#"
            puts $fid "tic_wd $tic_wd"

            #	tic_ht
            puts $fid "#"
            puts $fid "#   The tic_ht parameter determines the height of a"
            puts $fid "#   tic mark on the x axis, in pixels"
            puts $fid "#"
            puts $fid "tic_ht $tic_ht"

            #	xlabel_above
            puts $fid "#"
            puts $fid "#   The xlabel_above parameter sets the x axis labels"
            puts $fid "#   above the axis line (default is 0, meaning below"
            puts $fid "#   the axis line)"
            puts $fid "#"
            puts $fid "xlabel_above $xlabel_above"

            #	ylabel_right
            puts $fid "#"
            puts $fid "#   The ylabel_right parameter sets the y axis labels"
            puts $fid "#   right of the axis line (default is 0, meaning left"
            puts $fid "#   of the axis line)"
            puts $fid "#"
            puts $fid "ylabel_right $ylabel_right"

            #	xlabel_xshift
            puts $fid "#"
            puts $fid "#   The xlabel_xshift parameter adjusts the location"
            puts $fid "#   of the x tic label left or right of the x tic mark"
            puts $fid "#   in pixels (default is 0)"
            puts $fid "#"
            puts $fid "xlabel_xshift $xlabel_xshift"

            #	xlabel_yshift
            puts $fid "#"
            puts $fid "#   The xlabel_yshift parameter adjusts the location"
            puts $fid "#   of the x tic label above or below of the x tic mark"
            puts $fid "#   in pixels (default is -6)"
            puts $fid "#"
            puts $fid "xlabel_yshift $xlabel_yshift"

            #	ylabel_xshift
            puts $fid "#"
            puts $fid "#   The ylabel_xshift parameter adjusts the location"
            puts $fid "#   of the y tic label left or right of the y tic mark"
            puts $fid "#   in pixels (default is -6)"
            puts $fid "#"
            puts $fid "ylabel_xshift $ylabel_xshift"

            #	ylabel_yshift
            puts $fid "#"
            puts $fid "#   The ylabel_yshift parameter adjusts the location"
            puts $fid "#   of the y tic label above or below of the y tic mark"
            puts $fid "#   in pixels (default is 0)"
            puts $fid "#"
            puts $fid "ylabel_yshift $ylabel_yshift"

            #	use_canvas_scrollbars
            puts $fid "#"
            puts $fid "#   The use_canvas_scrollbars flag turns on (1) or off"
            puts $fid "#   (0) the scrollbars.  Default is on."
            puts $fid "#"
            puts $fid "use_canvas_scrollbars $use_canvas_scrollbars"

            #	canvas_magnification
            puts $fid "#"
            puts $fid "#   The canvas_magnification factor determines how much"
            puts $fid "#   more canvas extends beyond the frame size that can"
            puts $fid "#   scrolled with the scrollbars.  A factor of 2 would"
            puts $fid "#   mean that the scrollable canvas is twice as large"
            puts $fid "#   as the frame."
            puts $fid "#"
            puts $fid "canvas_magnification $canvas_magnification"

            #	max_ortho_lines
            puts $fid "#"
            puts $fid "#   The number of lines (in each direction) to be drawn"
            puts $fid "#   with the Ortho Lines tool"
            puts $fid "#"
            puts $fid "max_ortho_lines $max_ortho_lines"

            #	aspect_ortho_lines
            puts $fid "#"
            puts $fid "#   The Aspect Ratio of the ortho line grid drawn"
            puts $fid "#   with the Ortho Lines tool"
            puts $fid "#"
            puts $fid "aspect_ortho_lines $aspect_ortho_lines"

            #   map file
            if { [string length $Curr_Map_File] > 0 } {
                if { $save_map_file_flag eq 1 } {
                    puts $fid "#"
                    puts $fid "#   The name of a map file in the current path"
                    puts $fid "#   to load"
                    puts $fid "#"
                    puts $fid "map_file [file tail $Curr_Map_File]"
                }
            }

            close $fid
        } else {
            Error_Message\
                "Cannot write to config file $filename--check permissions"
            return
        }
    }
}
#	procedure	--	ConfigFile
#	----------------------------------------------------------------------



#   --------------------------------------------------------------------------
#   Procedure -- Read_Ascii_File
#
#   Description
#
#       Reads in an entire ASCII file.  Each line is read and concatenated
#       into one, long string.
#
#   Arguments
#
#       filename
#
#   Returns
#
#       Returns the file as one, long string.
#
#  --------------------------------------------------------------------------
proc Read_Ascii_File { filename } {

    #
    #   catch a non-existent file
    #
    if { [string length $filename] == 0 || \
         [file exists $filename] == 0 } {
        Error_Message "file $filename does not exist"
        return ""
    }

    set whole_file_string ""

    #
    #	read
    #
    set fid [open $filename "r"]
    if { $fid >= 0 } {

        #
        #	loop through all lines in the file
        #
        while { [eof $fid] ne 1 } {
            set fline [gets $fid]

            #
            #   concatenate all strings
            #
            set whole_file_string "$whole_file_string $fline"
        }
        # while -- eof

    } else {
        Error_Message "could not open file $filename"
        return ""
    }

    close $fid
    return $whole_file_string
}
#   proc -- Read_Ascii_File
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   --------------------------------------------------------------------------
#   Procedure -- Read_Data_File
#
#   Description
#
#       Reads a file expected to be a data file of paired columns.  Each
#       pair is interpreted as a set of parametric, x-y, data coordinates.
#       Each line in the file is received as one row with 2*n columns.
#       Each row becomes one array element in the output, "Data_Array".
#
#   Arguments
#
#       filename
#
#   Returns
#
#       Void.  The collected data is set into a global array, "Data_Array".
#
#  --------------------------------------------------------------------------
proc Read_Data_File { filename } {
global Data_Array

    #
    #   catch a non-existent file
    #
    if { [string length $filename] == 0 || \
         [file exists $filename] == 0 } {
        Error_Message "file $filename does not exist"
        return ""
    }

    if { [info exist Data_Array] == 1 } {
        unset Data_Array
    }

    set whole_file_string ""

    #
    #	read
    #
    set fid [open $filename "r"]
    if { $fid >= 0 } {

        #
        #	loop through all lines in the file
        #
        set file_line_count 1
        while { [eof $fid] ne 1 } {
            set fline [gets $fid]

            #
            #   The file line becomes one element in an array
            #
            set Data_Array($file_line_count) $fline
            incr file_line_count
        }
        # while -- eof

    } else {
        Error_Message "could not open file $filename"
        return ""
    }

    close $fid
}
#   proc -- Read_Data_File
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------



#   --------------------------------------------------------------------------
#   Procedure -- Separate_Map_Strings
#
#   Description
#
#       Takes a string of one or more map strings delimited by ";" chars.
#       example:
#           <map_spec_1> ; <map_spec_2> ; ...
#
#   Arguments
#
#   Returns
#
#       Returns the map_strings separated from the ";" characters and
#       and each string now a list element.  Each string is trimmed of
#       whitespace:
#
#           { <map_string_1>  <map_string_2>  .... }
#
#  --------------------------------------------------------------------------
proc Separate_Map_Strings { all_map_strings } {

    set map_list [split $all_map_strings ";"]
    set map_num [llength $map_list]
    for {set m 0} {$m < $map_num} {incr m} {
        set map_str2 [lindex $map_list $m]
        set map_str3 [string trimleft [string trimright $map_str2]]
        if { [string length $map_str3] == 0 } {
            continue
        } else {
            lappend map_string_list $map_str3
        }
    }
    return $map_string_list
}
#   proc -- Separate_Map_Strings
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   --------------------------------------------------------------------------
#   Procedure -- Convert_Map_to_Eval
#
#   Description
#
#       Convert the string found in a mapping spec file (or plot file) into
#       an eval string.  This does not include the loop wrapper nor the
#       pixel conversion and Draw_MPoint function.  It is just enough code
#       to calculate "wx" and "wy".
#
#
#   Arguments
#
#       map_string
#
#           The string received from the map file or plot file.
#
#       add
#
#           Flag that tells the function whether to make this make a permanant
#           map in the global list (add==1) or only a temporary map that
#           affects only the current plot (add==0).
#
#   Returns
#
#       Returns the eval string for "wx" and "wy".
#
#  --------------------------------------------------------------------------
proc Convert_Map_to_Eval { map_string add } {
global mapeval_list maptype_list mapname_list
global max_mapping_mode
global mnMapping_Userfunction
global Last_Predefined_Map_Mode

    set maptype "User Function"
    set mapname "function"

    set map_components [Split_Plot_String $map_string]
    set m_variable_list [lindex $map_components 0]
    set m_expression_list [lindex $map_components 1]
    if { [info exist m_variable_list] == 0 } {
        Error_Message "no variables found in map string $map_string"
        return
    }
    if { [info exist m_expression_list] == 0 } {
        Error_Message "no expressions found in map string $map_string"
        return
    }
    #
    #   More error checking
    #
    set m_kmax [llength $m_variable_list]
    set m_jmax [llength $m_expression_list]
    if { $m_kmax != $m_jmax } {
        #
        #   Error--syntax on one of the equations
        #
        Error_Message "variables not equal to expressions in map $map_string"
        return
    }
    set m_etype_list [Assign_Expression_Types $m_variable_list \
                     $m_expression_list]

    #
    #   Make the command string
    #
    set map_func_string ""
    for {set k 0} {$k < $m_kmax} {incr k} {
        #
        #   Get the next variable and its expression
        #
        set varstr [lindex $m_variable_list $k]
        set exprstr [lindex $m_expression_list $k]
        set etype [lindex $m_etype_list $k]

        #
        #   assemble a command string which will be executed to create
        #   the plot by "eval"
        #
        if { $etype eq 1 } {
            #
            #   Add a command string
            #   String will set a constant to a variable
            #
            set map_func_string "$map_func_string set $varstr $exprstr\n"
        } elseif { $etype eq 2 } {
            #
            #   Add a command string
            #   String will set a variable from an evaluation
            #
            set map_func_string "$map_func_string set $varstr \[expr $exprstr\]\n"
        } elseif { $etype eq 7 } {
            #
            #   Add a command string
            #
            set maptype $exprstr
        } elseif { $etype eq 8 } {
            #
            #   Add a command string
            #
            set mapname $exprstr
        } elseif { $etype eq 9 } {
            #
            #   Add a command string
            #   String will set a variable from an evaluation
            #
            set map_func_string "$map_func_string set $varstr \[$exprstr\]\n"
        }
    }
    # for -- variables

    if { $add > 0 } {
        #
        #   Add this map function to the global list
        #
        lappend mapeval_list $map_func_string
        lappend maptype_list $maptype
        lappend mapname_list $mapname
        incr max_mapping_mode

        #
        #   Add this map function to the menu
        #
        $mnMapping_Userfunction add command -label $mapname -command \
        {
            #
            #   This command to be evaluated when the menu is activated
            #   So the mapping index must be searched.
            #
            set my_mapname [$mnMapping_Userfunction entrycget active -label]
            set curr_index [$mnMapping_Userfunction index $my_mapname]
            set mapping_mode [expr $curr_index + $Last_Predefined_Map_Mode]
            # Info_Message "map_mode for $my_mapname is $mapping_mode"
            Reset_Mapping
        }
    }

    return $map_func_string
}
#   proc -- Convert_Map_to_Eval
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   --------------------------------------------------------------------------
#   Procedure -- Split_Range_String
#
#   Description
#
#   Expects the parametric range string to have the format
#   
#        <initial_value>:<increment value>:<final_value>
#
#   Arguments
#
#       s_string
#
#           The range string that was parsed out of a plot string.  The range
#           string must be of the format
#                t0:dt:tmax
#           where t0 = the initial value of the parameteric variable, dt is
#           the increment value, and tmax is the final value.
#
#   Returns
#
#       Returns a list of format:
#
#       { <initial_value> <increment> <final_value> }
#
#  --------------------------------------------------------------------------
proc Split_Range_String { s_string } {

    if { [string length $s_string] == 0 } {
        Error_Message "Split_Range_String: empty function string"
        return
    }

    set seps [list ":"]
    set s_list [split $s_string $seps]
    set s_length [llength $s_list]

    for {set i 0} {$i < $s_length} {incr i} {
        #
        #   parse elements
        #
        set s_whole [lindex $s_list $i]
        set elem_string [string trimleft [string trimright $s_whole]]
        if { [string length $elem_string] == 0 } {
            continue
        }

        lappend ret_list $elem_string
    }
    # for -- s_list

    return $ret_list
}
#   proc -- Split_Range_String
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------



#   --------------------------------------------------------------------------
#   Procedure -- Split_Plot_String
#
#   Description
#
#       Parses a plot string into separate equations that are delimited by
#       commas;  then parses each of the equations by splitting on the
#       equals sign:  the value to the left of the equals becomes the variable,
#       and the value to the right of the equals sign becomes the expression.
#
#   Arguments
#
#       plot_string
#
#           A string taken from a plot file.
#
#   Returns
#
#       Returns a list of format:
#
#           { $variable_list $expression_list }
#
#  --------------------------------------------------------------------------
proc Split_Plot_String { plot_string } {

    set seps2 [list "\|"]
    set seps3 [list "="]
    #
    #   split the separate statements (separated by commas) within the
    #   plot string
    #
    set statement_list [split $plot_string $seps2]

    set num_statements [llength $statement_list]
    for {set j 0} {$j < $num_statements} {incr j} {

        #
        #   Get next statement in this plot
        #
        set statement_whole [lindex $statement_list $j]
        set statement_string [string trimleft [string trimright\
             $statement_whole]]
        if { [string length $statement_string] == 0 } {
            continue
        }

        #
        #   split each statement on an "=" sign
        #
        set equation_list [split $statement_string $seps3]
        set parts 0
        set variable_name ""

        set num_eqparts [llength $equation_list]
        for {set k 0} {$k < $num_eqparts} {incr k} {

            #
            #   Get next part in this equation
            #
            set eqpart_whole [lindex $equation_list $k]
            set eqpart_string [string trimleft [string trimright\
                 $eqpart_whole]]
            if { [string length $eqpart_string] == 0 } {
                continue
            }


            if { $parts eq 0 } {
                #
                #   first element should be the identifier
                #
                lappend variable_list $eqpart_string
                # Info_Message "appending $eqpart_string to variables"


                set parts 1
            } else {
                #
                #   next element should be the expression
                #

                #
                #   create a new variable with the "variable" statement
                #
                lappend expression_list $eqpart_string
                # Info_Message "appending $eqpart_string to expressions"
            }

        }
        # for -- num_eqparts

    }
    # for -- num_statements

    set retval [list $variable_list $expression_list]
    return $retval
}
#   proc -- Split_Plot_String
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   --------------------------------------------------------------------------
#   Procedure -- Assign_Expression_Types
#
#   Description
#
#       Assigns enumerated types to each expression parsed out from a plot
#       string.
#
#   Arguments
#
#       variable_list
#
#           The list of variable names parsed out of a plot string
#
#       expression_list
#
#           The list of expressions parsed out of a plot string
#
#   Returns
#
#       Returns a list of types
#
#  --------------------------------------------------------------------------
proc Assign_Expression_Types { variable_list expression_list } {

    set kmax [llength $variable_list]
    #
    #   Loop through the variables -- identify equations
    #   Evaluate the constants first
    #
    for {set k 0} {$k < $kmax} {incr k} {
        #
        #   Get the next variable and its expression
        #
        set varstr [lindex $variable_list $k]
        set exprstr [lindex $expression_list $k]

        #
        #   Find the type of variable
        #

        if { $varstr eq "t" } {
            lappend etype_list 3
            continue
        }

        #
        #   Find the type of variable
        #
        if { $varstr eq "x" } {
            lappend etype_list 4
            continue
        }

        if { $varstr eq "y" } {
            lappend etype_list 5
            continue
        }

        if { $varstr eq "color" } {
            lappend etype_list 6
            continue
        }

        if { $varstr eq "type" } {
            lappend etype_list 7
            continue
        }

        if { $varstr eq "name" } {
            lappend etype_list 8
            continue
        }

        if { [regexp {([\+\-0-9.]*[0-9]+)([eE]+[\+\-]*[0-9]+)*}\
                      $exprstr] eq 1 } {
            if { [regexp {[\*\/\+\-]+} $exprstr] eq 1 } {
                #
                #   not a constant -- has an arithmetic symbol
                #
                lappend etype_list 2
                continue
            } elseif { [regexp {abs} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {acos} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {asin} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {atan} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {atan2} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {ceil} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {cos} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {cosh} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {double} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {exp} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {floor} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {fmod} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {hypot} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {int} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {log} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {log10} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {pow} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {rand} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {round} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {sin} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {sinh} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {sqrt} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {srand} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {tan} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {tanh} $exprstr] eq 1 } {
                lappend etype_list 2
                continue
            } elseif { [regexp {Max} $exprstr] eq 1 } {
                #
                #   zmapper-defined procedure
                #
                lappend etype_list 9
                continue
            } elseif { [regexp {Min} $exprstr] eq 1 } {
                #
                #   zmapper-defined procedure
                #
                lappend etype_list 9
                continue
            } else {
                #
                #   A number
                #
                #
                #   Create such a variable and assign its value
                #
                variable $varstr $exprstr
                lappend etype_list 1
                continue
            }
        }

        #
        #   default -- an expression to evaluate
        #
        lappend etype_list 2

    }
    # for -- variables

    return $etype_list
}
#   proc -- Assign_Expression_types
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   --------------------------------------------------------------------------
#   Procedure -- Load_Map_Function_String
#
#   Description
#
#       Load the map functions that were read from the map file.  Parse and
#       separate each map string, convert each map string into an eval
#       function.
#
#   Arguments
#
#       total_map_string -- a string containing the map function read from
#       the file.
#
#   Returns
#
#       void
#
#   --------------------------------------------------------------------------
proc Load_Map_Function_String { total_map_string } {
global max_mapping_mode Last_Predefined_Map_Mode
global mapeval_list maptype_list mapname_list
global mnMapping_Userfunction

    #
    #   Clear the last map list (if any) before the new maps are read
    #
    #   clear the menu
    #
    set del_index_1 $Last_Predefined_Map_Mode
    set del_index_2 $max_mapping_mode
    $mnMapping_Userfunction delete $del_index_1 $del_index_2
    #
    #   clear the map strings
    #
    set istart $Last_Predefined_Map_Mode
    if { [info exist mapeval_list] } {
        for {set i $istart} {$i < $max_mapping_mode} {incr i} {
            set mapeval_list [lreplace $mapeval_list $Last_Predefined_Map_Mode $Last_Predefined_Map_Mode]
        }
    }
    if { [info exist maptype_list] } {
        for {set i $istart} {$i < $max_mapping_mode} {incr i} {
            set maptype_list [lreplace $maptype_list $Last_Predefined_Map_Mode $Last_Predefined_Map_Mode]
        }
    }
    if { [info exist mapname_list] } {
        for {set i $istart} {$i < $max_mapping_mode} {incr i} {
            set mapname_list [lreplace $mapname_list $Last_Predefined_Map_Mode $Last_Predefined_Map_Mode]
        }
    }
    set max_mapping_mode $Last_Predefined_Map_Mode

    #
    #   "Compile" and add the new map list
    #
    set mapstr_list [Separate_Map_Strings $total_map_string]
    set num_maps [llength $mapstr_list]
    for {set i 0} {$i < $num_maps} {incr i} {

        #
        #   Get next map string
        #
        set map_whole [lindex $mapstr_list $i]

        #
        #   Convert this string into the corresponding "eval" form
        #   and insert it into the list of maps.
        #
        set map_string [Convert_Map_to_Eval $map_whole 1]
    }
}
#   proc -- Load_Map_Function_String
#   --------------------------------------------------------------------------
#   --------------------------------------------------------------------------



#   --------------------------------------------------------------------------
#   Procedure -- Make_Dynamic_Plot_Command
#
#   Description
#
#       Creates a string with Tcl commands to be passed to eval in order to
#       plot a user-defined function.  The string that was read from the file
#       must be processed before calling this function--it must be parsed
#       into separate lists for the variable names and the corresponding
#       expressions to calculate.
#
#       For example, if the user-defined function was read from the file as
#           pi = 3.1416, q = 2 * $pi,
#           t = 0 : 0.01 : $q,
#           x = 2*cos($t), y = sin($t)
#
#       The variable list should contain,
#           { "pi" "q" "t" "x" "y" }
#       The expression list should contain
#           { "3.1416"  "2 * $pi"  "0 : 0.01 : $q"  "2*cos($t)"  "sin($t)" }
#       The etype list should contain enumerated identifiers
#           { 1  2  3  4  5 }
#       see "Assign_Expression_Types" procedure.
#
#   Arguments
#
#       variable_list
#
#           A list containing the variable names for each expression.
#
#       expression_list
#
#           A list containing each expression.
#
#       etype_list
#
#           A list containing the type of each expression.
#
#       mapstr
#
#           A string containing the the "eval function" for calculating a
#           mapping that corresponds to the user-defined function.  This
#           string is allowed to be empty.
#
#   Returns
#
#  --------------------------------------------------------------------------
proc Make_Dynamic_Plot_Command { variable_list expression_list etype_list mapstr } {

    set kmax [llength $variable_list]
    #
    #   Create a dynamic function -- a string that we be passed to
    #   eval
    #
    set dyn_func_string ""
    set dyn_func_string "$dyn_func_string global tp_Mu tp_Bu tp_Mv tp_Bv\n"
    set dyn_func_string "$dyn_func_string global curr_draw_color\n"
    set dyn_func_string "$dyn_func_string set save_draw_color \$curr_draw_color\n"

    set dyn_func_string\
     "$dyn_func_string if \{ \[info exist x_list\] != 0 \} \{ unset x_list \}\n"
    set dyn_func_string\
     "$dyn_func_string if \{ \[info exist y_list\] != 0 \} \{ unset y_list \}\n"

    #
    #   Each data point -- Loop through the variables
    #   Assign a command string based on each type of expression
    #
    for {set k 0} {$k < $kmax} {incr k} {
        #
        #   Get the next variable and its expression
        #
        set varstr [lindex $variable_list $k]
        set exprstr [lindex $expression_list $k]
        set etype [lindex $etype_list $k]

        #
        #   assemble a command string which will be executed to create
        #   the plot by "eval"
        #
        if { $etype eq 1 } {
            #
            #   Add a command string
            #   String will set a constant to a variable
            #
            set dyn_func_string "$dyn_func_string set $varstr $exprstr\n"
        } elseif { $etype eq 2 } {
            #
            #   Add a command string
            #   String will set a variable from an evaluation
            #
            set dyn_func_string "$dyn_func_string set $varstr \[expr $exprstr\]\n"
        } elseif { $etype eq 3 } {
            #
            #   Add a command string
            #   This is the loop range
            #
            set range_list [Split_Range_String $exprstr]
            set ival [lindex $range_list 0]
            set incval [lindex $range_list 1]
            set fnval [lindex $range_list 2]

            set dyn_func_string "$dyn_func_string for \{set t $ival\} \
                \{\$t < $fnval\} \{set t \[expr \$t + $incval\]\} \{\n"
        } elseif { $etype eq 4 } {
            #
            #   Add a command string
            #   String will set a variable from an evaluation
            #
            set dyn_func_string "$dyn_func_string set $varstr \[expr $exprstr\]\n"
        } elseif { $etype eq 5 } {
            #
            #   Add a command string
            #   String will set a variable from an evaluation
            #
            set dyn_func_string "$dyn_func_string set $varstr \[expr $exprstr\]\n"
        } elseif { $etype eq 6 } {
            #
            #   Add a command string
            #
            set dyn_func_string "$dyn_func_string set curr_draw_color #$exprstr\n"
        }
    }
    # for -- variables

    #
    #   Add a command string to convert the math-x coordinate to a pixel
    #   coordinate
    #
    set dyn_func_string "$dyn_func_string set px \[Pixel_x \$x \$tp_Mu \$tp_Bu\]\n"
    #
    #   Add a command string to convert the math-y coordinate to a pixel
    #   coordinate
    #
    set dyn_func_string "$dyn_func_string set py \[Pixel_y \$y \$tp_Mv \$tp_Bv\]\n"
    #
    #   Add a command string to plot the point
    #
    set dyn_func_string "$dyn_func_string Draw_Point \$px \$py \$canv\n"

    #
    #   If a mapping is specified, add it to the loop
    #
    if { [string length $mapstr] > 0 } {
        #
        #   Tell the code to save its "t" values for later doing the map
        #
        set dyn_func_string "$dyn_func_string lappend x_list \$x\n"
        set dyn_func_string "$dyn_func_string lappend y_list \$y\n"
    }

    #
    #   put the final curly bracket on the for loop
    #
    set dyn_func_string "$dyn_func_string \}\n"
    set dyn_func_string "$dyn_func_string Unset_LastMouse\n"
    set dyn_func_string "$dyn_func_string set curr_draw_color \$save_draw_color\n"

    #
    #   If we want mapping, now add it
    #
    if { [string length $mapstr] > 0 } {
        #
        #   Retreive the lists that saved the x,y values
        #
        set dyn_func_string "$dyn_func_string set tmx \[llength \$x_list\]\n"
        set dyn_func_string "$dyn_func_string set tk 0\n"
        #
        #   loop through the x,y values again to do the mapping
        #
        set dyn_func_string "$dyn_func_string while \{ \$tk < \$tmx \} \{\n"
        set dyn_func_string "$dyn_func_string set x \[lindex \$x_list \$tk\]\n"
        set dyn_func_string "$dyn_func_string set y \[lindex \$y_list \$tk\]\n"
        #
        #   insert the map string -- math setup
        #
        set dyn_func_string "$dyn_func_string $mapstr\n"
        #
        set dyn_func_string "$dyn_func_string set px \[Pixel_x \$wx \$tp_Mu \$tp_Bu\]\n"
        set dyn_func_string "$dyn_func_string set py \[Pixel_y \$wy \$tp_Mv \$tp_Bv\]\n"
        set dyn_func_string "$dyn_func_string Draw_MPoint \$px \$py \$canv\n"
        set dyn_func_string "$dyn_func_string incr tk\n"
        #
        #   insert final curly bracket
        #
        set dyn_func_string "$dyn_func_string \}\n"
        set dyn_func_string "$dyn_func_string Unset_LastMouse\n"
    }

    return $dyn_func_string
}
#   proc -- Make_Dynamic_Plot_Command
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   --------------------------------------------------------------------------
#   Procedure -- Parse_Exec_Function_String
#
#   Description
#
#       Given the string read from the functions file, parses and plots each
#       of the functions (one plot between each set of [] delimiters).
#
#   Arguments
#
#       func_string
#
#           The string containing all the user-defined functions read from
#           the function file.
#
#       canv
#
#           The canvas widget path.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc Parse_Exec_Function_String { func_string canv } {
global mapping_mode
global PI

    if { [string length $func_string] == 0 } {
        Error_Message "empty function string"
        return
    }

    set old_cursor [$canv cget -cursor]
    $canv config -cursor watch
    # Info_Message "func_string= $func_string"

    set seps1 [list "\[\]"]

    #
    #   Split the whole string into separate plot statements:
    #   Each plot statement is contained within "[" and "]" characters.
    #
    set plot_list [split $func_string $seps1]

    set num_plots [llength $plot_list]
    for {set i 0} {$i < $num_plots} {incr i} {

        #
        #   clean up the previous loop's lists
        #
        if { [info exist variable_list ] != 0 } {
            unset variable_list
        }
        if { [info exist expression_list ] != 0 } {
            unset expression_list
        }
        set found_map 0

        #
        #   Get next plot string
        #
        set plot_whole [lindex $plot_list $i]
        set plot_string [string trimleft [string trimright $plot_whole]]
        if { [string length $plot_string] == 0 } {
            continue
        }

        #
        #   The plot string might have a mapping string at the end
        #   separated by a ";"
        #
        set map_string ""
        if { [regexp {;(.*)} $plot_string map_str] eq 1 } {
            #
            #   We have the portion to the right of the ";" in the
            #   map_str variable.  We'll Separate_Map_Strings to clean
            #   up the ";" and whitespace.
            #
            set tmp_map_string [lindex [Separate_Map_Strings $map_str] 0]
            if { $mapping_mode > 0 } {
                #
                #   Put the map spec into an eval string
                #   This will be the part inside the loop
                #   The code that will insert the loop code and point plot
                #   is in Make_Dynamic_Plot_Command.
                #
                set map_string [Convert_Map_to_Eval $tmp_map_string 0]
                set found_map 1
            }
            # Info_Message "Found map string $map_string"
            #
            #   Remove the map_string part from plot_string so that the
            #   remaining function plot code does not see the mapping string.
            #
            set tmp_string $plot_string
            regsub {;(.*)} $tmp_string "" plot_string 
        }
        if { $mapping_mode > 0 && [string length $map_string] == 0 } {
            #
            #   If the function spec in the file did not include a map spec,
            #   use the default map
            #
            set map_string [Get_Curr_Map_EvalString]
        }
        #   end code handling the map spec

        #
        #   Parse the current plot (between one set of "[" and "]") into a
        #   variable list and en expression list
        #
        set plot_components [Split_Plot_String $plot_string]

        #
        #   Retreive the return values: variable list and expression list
        #
        set variable_list [lindex $plot_components 0]
        set expression_list [lindex $plot_components 1]

        #
        #   Parsing is done.
        #   Begin to evaluate the statements
        #   Some error checking
        #
        if { [info exist variable_list] == 0 } {
            Error_Message "no variables found in $plot_string"
            $canv config -cursor $old_cursor
            return
        }
        if { [info exist expression_list] == 0 } {
            Error_Message "no expressions found in $plot_string"
            $canv config -cursor $old_cursor
            return
        }

        #
        #   More error checking
        #
        set kmax [llength $variable_list]
        set jmax [llength $expression_list]
        if { $kmax != $jmax } {
            #
            #   Error--syntax on one of the equations
            #
            Error_Message "variables not equal to expressions in $plot_string"
            continue
        }

        #
        #   Identify the type of each expression
        #
        set etype_list [Assign_Expression_Types $variable_list $expression_list]

        #
        #   Assemble the variable names and expressions into a plot command
        #   string that will create the plot.
        #   Notice we re-insert the "map_string" created above so that a map
        #   can (if included) be plotted at the same time.
        #
        set dyn_func_string [Make_Dynamic_Plot_Command $variable_list \
                    $expression_list $etype_list $map_string]

        # Info_Message "$dyn_func_string"

        #
        #   "Call" the dynamic function
        #
        eval $dyn_func_string

        #   Continue on to the next plot string
    }
    # for -- num_plots

    $canv config -cursor $old_cursor
}
#   proc -- Parse_Exec_Function_String
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#  --------------------------------------------------------------------------
#   Procedure -- Open_File
#
#   Description
#
#       Procedure to open a file depending on what the current figure mode
#       happens to be.  If the figure mode is Function, then the file is
#       expected to contain function definitions;  if the figure mode is
#       Data, then the file is expected to be columns of data to plot.
#
#   Arguments
#
#       fullpath_to_file -- string containing the file name, including the
#       full directory path.
#
#       canv -- handle to the canvas on which we are drawing.
#
#   Returns
#
#       Void
#
#  --------------------------------------------------------------------------
proc Open_File { fullpath_to_file canv } {
global figure_mode

    #
    #  validate the input string
    #
    set err [catch { string length $fullpath_to_file } results]
    if { $err != 0 } {
       Error_Message "ERROR: Open_File: input file name string NULL"
       return -5
    }

    #
    #  validate the canv
    #
    set err [winfo exists $canv]
    if { $err == 0 } {
       Error_Message "ERROR: Open_File: input canv NULL"
       return -4
    }

    #
    #  Check if the file exists
    #
    if { [file exists $fullpath_to_file] eq 0 } {
       Error_Message "ERROR: file $fullpath_to_file does not exist"
       return -3
    }

    #
    #  Check if the file is a regular file
    #
    if { [file isfile $fullpath_to_file] eq 0 } {
       Error_Message "ERROR: file $fullpath_to_file is not a regular file"
       return -2
    }

    #
    #  Check if the file is readable
    #
    if { [file readable $fullpath_to_file] eq 0 } {
       Error_Message "ERROR: file $fullpath_to_file is not readable by the current user"
       return -1
    }


    if { $figure_mode eq 4 } {

        #
        #   Read the file into a text string
        #
        set file_string [Read_Ascii_File $fullpath_to_file]

        Parse_Exec_Function_String $file_string $canv

    }

    if { $figure_mode eq 5 } {

        #
        #   Read the file as data columns
        #
        Read_Data_File $fullpath_to_file

        #
        #   Data is set into a global array accessed by Draw_Data_Columns
        #
        Draw_Data_Columns $canv
        
    }

}
#  proc -- Open_File
#  ---------------------------------------------------------------------



#   --------------------------------------------------------------------------
#   Procedure -- Load_Map_File
#
#   Description
#
#       Procedure called from the Load Maps menu selection.
#
#   Arguments
#
#       fullpath_to_file -- string containing the file name, including the
#       full directory path.
#
#       canv -- handle to the canvas on which we are drawing.
#
#   Returns
#
#       Void.
#
#   --------------------------------------------------------------------------
proc Load_Map_File { fullpath_to_file canv } {
global figure_mode

    #
    #  validate the input string
    #
    set err [catch { string length $fullpath_to_file } results]
    if { $err != 0 } {
       Error_Message "ERROR: Load_Map_File: input file name string NULL"
       return -5
    }

    if { [string length $fullpath_to_file] eq 0 } {
        return 0
    }

    #
    #  validate the canv
    #
    set err [winfo exists $canv]
    if { $err == 0 } {
       Error_Message "ERROR: Load_Map_File: input canv NULL"
       return -4
    }

    #
    #  Check if the file exists
    #
    if { [file exists $fullpath_to_file] eq 0 } {
       Error_Message "ERROR: file $fullpath_to_file does not exist"
       return -3
    }

    #
    #  Check if the file is a regular file
    #
    if { [file isfile $fullpath_to_file] eq 0 } {
       Error_Message "ERROR: file $fullpath_to_file is not a regular file"
       return -2
    }

    #
    #  Check if the file is readable
    #
    if { [file readable $fullpath_to_file] eq 0 } {
       Error_Message "ERROR: file $fullpath_to_file is not readable by the current user"
       return -1
    }


    #
    #   Read the file into a text string
    #
    set file_string [Read_Ascii_File $fullpath_to_file]

    Load_Map_Function_String $file_string

    return 1
}
#   proc -- Load_Map_File
#   ---------------------------------------------------------------------
#   ---------------------------------------------------------------------




#	----------------------------------------------------------------------
#	hints:
#
#	-font {times 12 bold}
#
#	----------------------------------------------------------------------



#	-------------------------------
#	Create the Base Window
#	-------------------------------
#
#	Because of Tcl's packing mechanism, we need two frames: 1 for the
#	menubar and 1 for the Client area
#
frame .f_menubar -width $wd_MenuBar -height $ht_MenuBar -bg grey
frame .f_displaybar -width $wd_DisplayBar -height $ht_DisplayBar -bg grey
frame .f_displaybar2 -width $wd_DisplayBar2 -height $ht_DisplayBar2 -bg grey
frame .f_client -width $wd_Client -height $ht_Client -bg grey
pack .f_menubar .f_displaybar .f_displaybar2 -side top -fill x
pack .f_client  -fill both -expand true
# pack .f_client
pack propagate .f_client false;  # prevent client from shrinking




#	-------------------------------------
#	Create the Menu across the top
#	-------------------------------------

#
#	"File" menu button
#
menubutton .f_menubar.mbfile -text File -menu .f_menubar.mbfile.menu -relief raised
set mnFile [menu .f_menubar.mbfile.menu]
pack  .f_menubar.mbfile -side left -ipadx 3 -ipady 3
	#
	#	Sub-menu under File --  "New"
	#
	$mnFile add cascade -label "New" -menu $mnFile.new
	set mnFile_New [menu $mnFile.new]
		#
		#	menu items under "New"
		#
		$mnFile_New add command -label Vibration -command {\
            Info_Message "Not Implimented" }

	#
	#	Sub-menu under File --  "Open"
	#
	$mnFile add command -label "Open" -command {\
        #
        #   Actions to open a file
        #
        set temp_path [tk_getOpenFile \
            -initialdir $curr_file_folder -initialfile $curr_file_name -title "Open"]
        if {$temp_path eq ""} {
            # leave the old path
        } else {
            Set_Filename [file tail $temp_path]
            Open_File $curr_file_fullpath $curr_canvas
        }
    }

	#
	#	Sub-menu under File --  "Load"
	#
	$mnFile add command -label "Load Maps" -command {\
        #
        #   Actions to open a file
        #
        set temp_path [tk_getOpenFile \
            -initialdir $curr_file_folder -initialfile "" -title "Load Maps"]
        if {$temp_path eq ""} {
            # leave the old path
        } else {
            set load_return [Load_Map_File $temp_path $curr_canvas]
            if { $load_return > 0 } {
                #
                #   If the load was successfull, save the map name
                #
                set Curr_Map_File $temp_path
            }
        }
    }

    #
    #	File -- menu item -- "Save"
    #
    $mnFile add command -label Save -command {\
         Info_Message "Not Implimented" }
    #
    #	File -- menu item -- "Quit"
    #
    $mnFile add command -label Quit -command {\
        set respval [tk_messageBox -type yesno -title Quit -message\
               "Quit without saving?" -icon question]
        if { [string compare "yes" $respval] eq 0 }  {
            exit
        }
    }
    #
    #	File -- menu item -- "Exit"
    #
    $mnFile add command -label Exit -command {\
        set respval [tk_messageBox -type yesno -title Exit -message\
               "Save and Exit?" -icon question]
        if { [string compare "yes" $respval] eq 0 }  {
	        ConfigFile 1 $configfile_name 
            exit
        }
    }

#
#	"Edit" menu button
#
menubutton .f_menubar.mbedit -text Edit -menu .f_menubar.mbedit.menu -relief raised
set mnEdit [menu .f_menubar.mbedit.menu]
pack  .f_menubar.mbedit -side left -ipadx 3 -ipady 3
    #
    #	menu items under "Edit"
    #
    #	Clear All
    #
    $mnEdit add command -label "clear all" -command  {   
        set imax [llength $figures_list] 
        for {set i 0} {$i < $imax} {incr i} { 
            set lval [lindex $figures_list $i] 
            .f_client.c1 delete $lval  \
        }
        # clear the list 
        unset figures_list
    }
    #
    #	Disable Map File Save (temporary)
    #
    $mnEdit add checkbutton -variable save_map_file_flag \
     -onvalue 1 -offvalue 0 \
     -label "save map name" -command \
    {
    }

#
#	"View" menu button
#
menubutton .f_menubar.mbview -text View -menu .f_menubar.mbview.menu -relief raised
set mnView [menu .f_menubar.mbview.menu]
pack  .f_menubar.mbview -side left -ipadx 3 -ipady 3
    #
    #	Item under View --  "Center Canvas"
    #
    $mnView add command -label "Center Canvas" -command \
    {
        Center_Canvas
    }
    #
    #	Sub-menu under View --  "Scaling"
    #
    $mnView add cascade -label "Scaling" -menu $mnView.scaling
    set mnView_Scaling [menu $mnView.scaling]
        #
        #	menu items under "Scaling"
        #
        $mnView_Scaling add command -label "mag 1: 0.02/pixel" -command \
        { 
            set Scaling_last $Scaling
            set Scaling 0.02
            set coord_scaling "$Scaling units/pixel"
            Scale_Figures_1 [expr $Scaling_last / $Scaling]
            Reset_View
            Scale_Figures_2
        }
        #
        #	menu items under "Scaling"
        #
        $mnView_Scaling add command -label "mag 2: 0.01/pixel" -command \
        {
            set Scaling_last $Scaling
            set Scaling 0.01 
            set coord_scaling "$Scaling units/pixel"
            Scale_Figures_1 [expr $Scaling_last / $Scaling]
            Reset_View
            Scale_Figures_2
        }
        #
        #	menu items under "Scaling"
        #
        $mnView_Scaling add command -label "mag 3: 0.005/pixel" -command \
        {
            set Scaling_last $Scaling
            set Scaling 0.005 
            set coord_scaling "$Scaling units/pixel"
            Scale_Figures_1 [expr $Scaling_last / $Scaling]
            Reset_View
            Scale_Figures_2
        }
        #
        #	menu items under "Scaling"
        #
        $mnView_Scaling add command -label "mag User" -command \
        {
            set Scaling_last $Scaling
            set xval [GetValue "Enter units/pixel factor ($Scaling)"]
            if { [string length $xval] > 0 } {
                set Scaling [Abs $xval]
                set coord_scaling "$Scaling units/pixel"
                Scale_Figures_1 [expr $Scaling_last / $Scaling]
                Reset_View
                Scale_Figures_2
            }
        }
    #
    #	Sub-menu under View --  "Shift"
    #
    $mnView add cascade -label "Shift" -menu $mnView.shift
    set mnView_Shift [menu $mnView.shift]
        #
        #	menu items under "Shift"
        #
        $mnView_Shift add command -label "center x,y" -command \
        {
            set xcenter_last $xcenter
            set ycenter_last $ycenter
            set xcenter 0
            set ycenter 0
            Reset_View
            Move_Figures [expr $xcenter - $xcenter_last] [expr $ycenter - $ycenter_last]
        }
        $mnView_Shift add command -label "center x" -command \
        {
            set xcenter_last $xcenter
            set xcenter 0
            Reset_View
            Move_Figures [expr $xcenter - $xcenter_last] 0
        }
        $mnView_Shift add command -label "center y" -command \
        { 
            set ycenter_last $ycenter
            set ycenter 0
            Reset_View
            Move_Figures 0 [expr $ycenter - $ycenter_last]
        }
		#
		#	menu items under "Shift"
		#
		$mnView_Shift add command -label "shift x" -command \
        {
            set xval [GetValue "Enter x shift"]
            if { [string length $xval] > 0 } {
                set xcenter_last $xcenter
                set xcenter $xval
                Reset_View
                Move_Figures [expr $xcenter - $xcenter_last] 0
            }
        }
        #
        #	menu items under "Shift"
        #
        $mnView_Shift add command -label "shift y" -command \
        {
            set yval [GetValue "Enter y shift"]
            if { [string length $yval] > 0 } {
                set ycenter_last $ycenter
                set ycenter $yval
                Reset_View
                Move_Figures 0 [expr $ycenter - $ycenter_last]
            }
        }
    #
    #	Sub-menu under View --  "Axes"
    #
    $mnView add cascade -label "Axes" -menu $mnView.axes
    set mnView_Axes [menu $mnView.axes]
        #
        #	menu items under "Axes"
        #
        $mnView_Axes add checkbutton \
         -variable xaxis_mode -onvalue 1 -offvalue 0 \
         -label "x axis" -command \
        {
            Draw_Axes 
        }
        $mnView_Axes add checkbutton \
         -variable yaxis_mode -onvalue 1 -offvalue 0 \
         -label "y axis" -command \
        {
            Draw_Axes 
        }
        $mnView_Axes add checkbutton \
         -variable xtics_mode -onvalue 1 -offvalue 0 \
         -label "x tics" -command \
        {
            Draw_Axes 
        }
        $mnView_Axes add checkbutton \
         -variable ytics_mode -onvalue 1 -offvalue 0 \
         -label "y tics" -command \
        {
            Draw_Axes 
        }
        $mnView_Axes add checkbutton \
         -variable xlabels_mode -onvalue 1 -offvalue 0 \
         -label "x labels" -command \
        {
            Draw_Axes 
        }
        $mnView_Axes add checkbutton \
         -variable ylabels_mode -onvalue 1 -offvalue 0 \
         -label "y labels" -command \
        {
            Draw_Axes 
        }




#
#	"Mapping" menu button
#
menubutton .f_menubar.mbmapping -text Mapping -menu .f_menubar.mbmapping.menu -relief raised
set mnMapping [menu .f_menubar.mbmapping.menu]
pack  .f_menubar.mbmapping -side left -ipadx 3 -ipady 3
    #
    #	Item under Mapping --  "None"
    #
    $mnMapping add command -label "None" -command \
    {
        set mapping_mode 0
        Reset_Mapping
    }
    #
    #	Sub-menu under Mapping --  "Inversion"
    #
    $mnMapping add command -label "Inversion" -command \
    {
        set mapping_mode 1
        Reset_Mapping
    }
    #
    #	Sub-menu under Mapping --  "ln(z)"
    #
    $mnMapping add command -label "ln(z)" -command \
    {
        set mapping_mode 2
        Reset_Mapping
    }
    #
    #	Sub-menu under Mapping --  "exp(z)"
    #
    $mnMapping add command -label "exp(z)" -command \
    {
        set mapping_mode 3
        Reset_Mapping
    }

    #
    #	Sub-menu under Mapping --  "User Function"
    #
    $mnMapping add cascade -label "User Function" -menu $mnMapping.userfunction
    set mnMapping_Userfunction [menu $mnMapping.userfunction]

#
#	"Tools" menu button
#
menubutton .f_menubar.mbtools -text Tools -menu .f_menubar.mbtools.menu -relief raised
set mnTools [menu .f_menubar.mbtools.menu]
pack  .f_menubar.mbtools -side left -ipadx 3 -ipady 3
    #
    #	Item under Tools -- "Freehand"
    #
    $mnTools add command -label "Freehand" -command \
    {
        set figure_mode 1
        Reset_FigureMode
    }
    $mnTools add command -label "Lines" -command \
    {
        set figure_mode 2
        Reset_FigureMode
    }
    $mnTools add command -label "Ortho Lines" -command \
    {
        set figure_mode 3
        Reset_FigureMode
    }
    $mnTools add command -label "Functions" -command \
    {
        set figure_mode 4
        Reset_FigureMode
    }
    $mnTools add command -label "Data" -command \
    {
        set figure_mode 5
        Reset_FigureMode
    }


#
#	"Options" menu button
#
menubutton .f_menubar.mboptions -text Options -menu .f_menubar.mboptions.menu -relief raised
set mnOptions [menu .f_menubar.mboptions.menu]
pack  .f_menubar.mboptions -side left -ipadx 3 -ipady 3
    #
    #	Sub-menu under Options --  "Pen Style"
    #
    $mnOptions add cascade -label "Pen Style" -menu $mnOptions.pen_style
    set mnOptions_PenStyle [menu $mnOptions.pen_style]
        #
        #	menu items under "Pen Style"
        #
        $mnOptions_PenStyle add checkbutton -variable pnstyle_1 \
         -onvalue 1 -offvalue 0 \
         -label "dots" -command \
        {
            Set_PenStyle 1
        }
        $mnOptions_PenStyle add checkbutton -variable pnstyle_2 \
         -onvalue 1 -offvalue 0 \
         -label "ovals" -command \
        {
            Set_PenStyle 2
        }
        $mnOptions_PenStyle add checkbutton -variable pnstyle_3 \
         -onvalue 1 -offvalue 0 \
         -label "lines" -command \
        {
            Set_PenStyle 3
        }
    #
    #	Sub-menu under Options --  "Ortho Lines"
    #
    $mnOptions add cascade -label "Ortho Line Options" -menu $mnOptions.ortho_lines
    set mnOptions_OrthoLines [menu $mnOptions.ortho_lines]
        #
        #	menu items under "Ortho Lines"
        #
        $mnOptions_OrthoLines add command -label "Number of Lines" -command \
        {
            set prompt_string "Enter Number of Lines ($max_ortho_lines)"
            set sval [GetValue $prompt_string]
            if { [string length $sval] > 0 } {
                set nval [expr int($sval)]
                if { $nval < 1 } {
                    Warning_Message "Value given ($sval) will be set to 1"
                    set nval 1
                }
                set max_ortho_lines $nval
            }
        }
        $mnOptions_OrthoLines add command -label "Aspect ratio" -command \
        {
            set prompt_string "Enter Aspect Ratio ($aspect_ortho_lines)"
            set sval [GetValue $prompt_string]
            if { [string length $sval] > 0 } {
                set dval [expr $sval * 1.0]
                if { $dval <= 0} {
                    Warning_Message "Value given ($sval) will be set to 0.1"
                    set dval 0.1
                } elseif { $dval > 1} {
                    Warning_Message "Value given ($sval) will be set to 1"
                    set dval 1
                }
                set aspect_ortho_lines $dval
            }
        }



#	---------------------------------------------------------------------
#	X,Y	position indicator
#	---------------------------------------------------------------------
label .f_displaybar.xbox -width 10 -anchor w -relief sunken -text "x :"
label .f_displaybar.ybox -width 10 -anchor w -relief sunken -text "y :"
label .f_displaybar.coordlabel -width 18 -anchor w -text $coord_units
label .f_displaybar.scalelabel -width 15 -anchor w -text $coord_scaling
label .f_displaybar.xrange -width 20 -anchor w -text $xrange
label .f_displaybar.yrange -width 20 -anchor w -text $yrange
pack .f_displaybar.xbox .f_displaybar.ybox .f_displaybar.coordlabel \
    .f_displaybar.scalelabel .f_displaybar.xrange .f_displaybar.yrange \
    -side left

label .f_displaybar2.mapmode -width 30 -anchor w -relief sunken\
      -text [Label_Mapmode $mapping_mode] -bg #ccccee
label .f_displaybar2.figmode -width 30 -anchor w -relief sunken\
      -text [Label_Figmode $figure_mode] -bg #cceecc
pack .f_displaybar2.mapmode .f_displaybar2.figmode -side left


# --------------------------------------------------------------------
#
# Canvas code
#
# --------------------------------------------------------------------
# ------- end canvas code ---------------------------------------------







#	--------------------------------------------------------------------
#	--------------------------------------------------------------------
#
#	Procedures
#
#	--------------------------------------------------------------------
#	--------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure_Name
#
#   Description
#
#   Arguments
#
#   Returns
#
#   -------------------------------------------------------------------------
#   procedure -- Procedure_Name
#   -------------------------------------------------------------------------
#   -------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Reset_Scaling
#
#   Description
#
#       Recalculate screen variables when the scaling is changed.
#
#   Arguments
#
#       Void.  Operates on global variable.
#
#   Returns
#
#       Void.
#
#   -------------------------------------------------------------------------
proc Reset_Scaling {  } {
global x_width y_height
global px_width py_height
global Scaling coord_scaling
global xmax xmin ymax ymin
global xrange yrange
global xcenter ycenter
global Ox Oy Ox_last Oy_last
global Have_Origin
global nx ny
global xdiv ydiv
global xdivscale ydivscale
global nx_p nx_m ny_p ny_m
global tp_Mu tp_Bu tp_Mv tp_Bv tp_Mx tp_Bx tp_My tp_By


    #
    #	screen dimensions given in the current scaling
    #
    set x_width [expr $px_width * $Scaling]
    set y_height [expr $py_height * $Scaling]
    #
    #	the limits of the screen in scaled units
    #
    set xmax [expr ($x_width / 2) + $xcenter]
    set xmin [expr $xcenter - ($x_width / 2)]
    set ymax [expr ($y_height / 2) + $ycenter]
    set ymin [expr $ycenter - ($y_height / 2)]
    set xrange "x range:  $xmin, $xmax"
    set yrange "y range:  $ymin, $ymax"


    #   -------------------------------------------------------------------------
    #   Transformation parameters
    set tp_Mu [Pixel_Mu $xmin $xmax $px_width]
    set tp_Bu [Pixel_Bu $xmin $xmax $px_width]
    set tp_Mv [Pixel_Mv $ymin $ymax $py_height]
    set tp_Bv [Pixel_Bv $ymin $ymax $py_height]
    set tp_Mx [Coord_Mx $xmin $xmax $px_width]
    set tp_Bx [Coord_Bx $xmin $xmax $px_width]
    set tp_My [Coord_My $ymin $ymax $py_height]
    set tp_By [Coord_By $ymin $ymax $py_height]
    #   -------------------------------------------------------------------------

    #
    #	pixel location of the axis origin
    #
    set Have_Origin 1
    if { [info exists Ox] } {
        set Ox_last $Ox
    }
    if { [info exists Oy] } {
        set Oy_last $Oy
    }
    set Ox [Pixel_x 0 $tp_Mu $tp_Bu]
    set Oy [Pixel_y 0 $tp_Mv $tp_Bv]
    if { $Ox > $px_width } {
        set Ox -1
    }
    if { $Oy > $py_height } {
        set Oy -1
    }
    if { $Ox < 0 } {
        set Have_Origin 0
    }
    if { $Oy < 0 } {
        set Have_Origin 0
    }
    #	------------  done pixel location of origin -----------

    #
    #   Adjust distance between tics according to scale
    #
    set xdiv [expr $Scaling / $xdivscale]
    set ydiv [expr $Scaling / $ydivscale]

    #
    #	tic divisions
    #	number of tics specifically on each side of axis
    #
    if { $xmax > 0 } {
        #
        #   xmax comprises the positive x axis
        #
        set nx_p [expr $xmax / $xdiv]
        if { $nx_p < 1 } {
            set nx_p 0
        }
    } else {
        #
        #   If xmax <= 0 then there is no positive axis
        #
        set nx_p 0
    }
    if { $xmin < 0 } {
        #
        #   xmin comprises the negative x axis
        #
        set nx_m [expr $xmin * (-1) / $xdiv]
        if { $nx_m < 1 } {
            set nx_m 0
        }
    } else {
        #
        #   If xmin >= 0 then there is no negative axis
        #
        set nx_m 0
    }
    if { $ymax > 0 } {
        #
        #   ymax comprises the positive y axis
        #
        set ny_p [expr $ymax / $ydiv]
        if { $ny_p < 1 } {
            set ny_p 0
        }
    } else {
        #
        #   If ymax <= 0 then there is no positive axis
        #
        set ny_p 0
    }
    if { $ymin < 0 } {
        #
        #   ymin comprises the negative y axis
        #
        set ny_m [expr $ymin * (-1) / $ydiv]
        if { $ny_m < 1 } {
            set ny_m 0
        }
    } else {
        #
        #   If ymin >= 0 then there is no negative axis
        #
        set ny_m 0
    }
    set nx [expr $nx_p + $nx_m]
    set ny [expr $ny_p + $ny_m]

    #
    #	reset the text in xrange and yrange indicators
    #
    .f_displaybar.scalelabel configure -text $coord_scaling
    .f_displaybar.xrange configure -text $xrange
    .f_displaybar.yrange configure -text $yrange

}
#	------------------ Reset_Scaling ----------------------------------



#	--------------------------------------------------------------------
#	Procedure:
#	Mouse_Get_Pixel
#	-----------
#	Returns the pixel x,y coordinates where the left mouse button was
#	clicked.
#	--------------------------------------------------------------------
#
proc Mouse_Get_Pixel { x y canv } {

   tk_messageBox -type yesnocancel -message "x pixel= $x, y pixel= $y"
	
}
#	---------------------- Mouse_Get_Pixel ------------------------------


#	--------------------------------------------------------------------
#	Procedure:
#	Inversion_Unit_Circle_X
#	---------------------
#	Calculates the x-position in the W plane for the Inversion in
#	the unit circle
#	--------------------------------------------------------------------
#
proc Inversion_Unit_Circle_X { x y } {


	set xw [expr $x / ($x*$x + $y*$y)]
	return $xw
}
#	----------------- Inversion_Unit_Circle_X --------------------------


#	--------------------------------------------------------------------
#	Procedure:
#	Inversion_Unit_Circle_Y
#	---------------------
#	Calculates the y-position in the W plane for the Inversion in
#	the unit circle
#	--------------------------------------------------------------------
#
proc Inversion_Unit_Circle_Y { x y } {


	set yw [expr $y / ($x*$x + $y*$y)]
	return $yw
}
#	----------------- Inversion_Unit_Circle_Y --------------------------



#   -------------------------------------------------------------------------
#   Procedure -- Draw_Mouse_Point
#
#   Description
#
#       Low-level procedure for plotting a point.  Used for drawing a point
#       or a small line segment between points.
#
#   Arguments
#
#       x -- the x location of the mouse (pixel coordinate)
#
#       y -- the y location of the mouse (pixel coordinate)
#
#       x0 -- the last x location of the mouse
#
#       y0 -- the last y location of the mouse
#
#       mcolor -- the color to use when plotting.
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_Mouse_Point { x y x0 y0 mcolor canv } {
global pen_style

    if { $pen_style == 1 } {
        #
        #   dots
        #
        set x1 $x
        set y1 $y
        set x2 $x
        set y2 $y
        set id [$canv create oval $x1 $y1 $x2 $y2\
                -outline $mcolor]
    } elseif { $pen_style == 2 } {
        #
        #   ovals
        #
        set x1 [expr $x-1]
        set y1 [expr $y-1]
        set x2 [expr $x+1]
        set y2 [expr $y+1]
        set id [$canv create oval $x1 $y1 $x2 $y2\
                -outline $mcolor]
    } elseif { $pen_style == 3 } {
        #
        #   lines
        #
        set id [$canv create line $x0 $y0 $x $y\
                -fill $mcolor -smooth 1]
    } else {
        set id -1
    }

    return $id
}
#   proc -- Draw_Mouse_Point
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMap_UnitCircle
#
#   Description
#
#       Low-level procedure (above Draw_Mouse_Point) for drawing a point on
#       the canvas and drawing the mapped point for the unit circle inversion.
#
#   Arguments
#
#       x -- the x location of the mouse (pixel coordinate)
#
#       y -- the y location of the mouse (pixel coordinate)
#
#       xs -- the x math coordinate
#
#       ys -- the y math coordinate
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc DrawMap_UnitCircle { x y xs ys canv } {
global last_x last_y last_px last_py
global tp_Mu tp_Bu tp_Mv tp_Bv
global figures_list

    #
    #	mapping unit circle
    #
    set den [expr $xs * $xs + $ys * $ys]
    if { $den eq 0 } {
        return
    }
    set wx [expr $xs / $den]
    set wy [expr $ys / $den]

    #
    #	conversion (of mapped points) to pixel coords
    #
    set px [Pixel_x $wx $tp_Mu $tp_Bu]
    set py [Pixel_y $wy $tp_Mv $tp_Bv]

    #
    #	draw the mapped points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_px] != 0 } {
        set id1 [Draw_Mouse_Point $px $py $last_px $last_py "red"\
                 $canv]
    }

    #
    #	draw the mouse points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_x] != 0 } {
        set id2 [Draw_Mouse_Point $x $y $last_x $last_y "lightblue"\
                 $canv]
    }

    #
    #	save each figure in a list
    #	for future manipulation (or clear)
    #
    if { [info exist id2] != 0 } {
        lappend figures_list $id1 $id2
    }

    set last_x $x
    set last_y $y
    set last_px $px
    set last_py $py
}
#   proc -- DrawMap_UnitCircle
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMap_Lnz
#
#   Description
#
#       Low-level procedure (above Draw_Mouse_Point) for drawing a point on
#       the canvas and drawing the mapped point for the ln(z) map.
#
#   Arguments
#
#       x -- the x location of the mouse (pixel coordinate)
#
#       y -- the y location of the mouse (pixel coordinate)
#
#       xs -- the x math coordinate
#
#       ys -- the y math coordinate
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc DrawMap_Lnz { x y xs ys canv } {
global last_x last_y last_px last_py
global tp_Mu tp_Bu tp_Mv tp_Bv
global figures_list

    #
    #	mapping ln(z)
    #
    set rp [expr hypot($xs, $ys)]
    set theta [expr atan2($ys, $xs)]
    set mp [Max $rp 0.000001]
    set wx [expr log($mp)]
    set wy $theta

    #
    #	conversion (of mapped points) to pixel coords
    #
    set px [Pixel_x $wx $tp_Mu $tp_Bu]
    set py [Pixel_y $wy $tp_Mv $tp_Bv]

    #
    #	draw the mapped points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_px] != 0 } {
        set id1 [Draw_Mouse_Point $px $py $last_px $last_py "red"\
                 $canv]
    }

    #
    #	draw the mouse points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_x] != 0 } {
        set id2 [Draw_Mouse_Point $x $y $last_x $last_y "lightblue"\
                 $canv]
    }

    #
    #	save each figure in a list
    #	for future manipulation (or clear)
    #
    if { [info exist id2] != 0 } {
        lappend figures_list $id1 $id2
    }

    set last_x $x
    set last_y $y
    set last_px $px
    set last_py $py
}
#   proc -- DrawMap_Lnz
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMap_Expz
#
#   Description
#
#       Low-level procedure (above Draw_Mouse_Point) for drawing a point on
#       the canvas and drawing the mapped point for the exp(z) map.
#
#   Arguments
#
#       x -- the x location of the mouse (pixel coordinate)
#
#       y -- the y location of the mouse (pixel coordinate)
#
#       xs -- the x math coordinate
#
#       ys -- the y math coordinate
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc DrawMap_Expz { x y xs ys canv } {
global last_x last_y last_px last_py
global tp_Mu tp_Bu tp_Mv tp_Bv
global figures_list

    #
    #	mapping exp(z)
    #
    set rp [expr exp($xs)]
    set wx [expr $rp * cos($ys)]
    set wy [expr $rp * sin($ys)]

    #
    #	conversion (of mapped points) to pixel coords
    #
    set px [Pixel_x $wx $tp_Mu $tp_Bu]
    set py [Pixel_y $wy $tp_Mv $tp_Bv]

    #
    #	draw the mapped points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_px] != 0 } {
        set id1 [Draw_Mouse_Point $px $py $last_px $last_py "red"\
                 $canv]
    }

    #
    #	draw the mouse points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_x] != 0 } {
        set id2 [Draw_Mouse_Point $x $y $last_x $last_y "lightblue"\
                 $canv]
    }

    #
    #	save each figure in a list
    #	for future manipulation (or clear)
    #
    if { [info exist id2] != 0 } {
        lappend figures_list $id1 $id2
    }

    set last_x $x
    set last_y $y
    set last_px $px
    set last_py $py
}
#   proc -- DrawMap_Expz
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMap_EvalMap
#
#   Description
#
#       Based on DrawMap_UnitCircle, but instead of the hard-coded unit circle
#       map, reads the current mapping mode and uses the eval map string that
#       was read from a user-defined map file.
#
#       Had to change a minor thing in approach: math coordinate arguments
#       must be "x" and "y" instead of "xs" and "ys".
#
#   Arguments
#
#       mx -- the x location of the mouse (pixel coordinate)
#
#       my -- the y location of the mouse (pixel coordinate)
#
#       x -- the x math coordinate
#
#       y -- the y math coordinate
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#  --------------------------------------------------------------------------
proc DrawMap_EvalMap { mx my x y canv } {
global last_x last_y last_px last_py
global tp_Mu tp_Bu tp_Mv tp_Bv
global figures_list

    #
    #	mapping from the user-defined map function
    #   calculates wx and wy
    #
    set mapstr [Get_Curr_Map_EvalString]
    eval $mapstr

    #
    #	conversion (of mapped points) to pixel coords
    #   (look here! wx and wy come from the "eval" operation on the map string)
    #
    set px [Pixel_x $wx $tp_Mu $tp_Bu]
    set py [Pixel_y $wy $tp_Mv $tp_Bv]

    #
    #	draw the mapped points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_px] != 0 } {
        set id1 [Draw_Mouse_Point $px $py $last_px $last_py "red"\
                 $canv]
    }

    #
    #	draw the mouse points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_x] != 0 } {
        set id2 [Draw_Mouse_Point $mx $my $last_x $last_y "lightblue"\
                 $canv]
    }

    #
    #	save each figure in a list
    #	for future manipulation (or clear)
    #
    if { [info exist id2] != 0 } {
        lappend figures_list $id1 $id2
    }

    set last_x $mx
    set last_y $my
    set last_px $px
    set last_py $py
}
#   proc -- DrawMap_EvalMap
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------




#   -------------------------------------------------------------------------
#   Procedure -- DrawMapLine_UnitCircle
#
#   Description
#
#       In figure_mode = "lines", and mapping_mode = "unit circle" this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapLine_UnitCircle { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse
    #
    #   When this function returns, it should drop down to Unset_LastMouse
    #   Since we must re-use last_x and last_y here.
    #
    set x $mouse_start_xs
    set y $mouse_start_ys
    set px [Pixel_x $x $tp_Mu $tp_Bu]
    set py [Pixel_y $y $tp_Mv $tp_Bv]
    for {set i 0} {$i < $num_segs} {incr i} {

        #
        #   Draw a point in the mapping
        #
        DrawMap_UnitCircle $px $py $x $y $canv

        #
        #   Update coordinates along the line
        #
        set x [expr $x + $x_seg]
        set y [expr $y + $y_seg]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
    }
}
#   proc -- DrawMapLine_UnitCircle
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMapLine_Lnz
#
#   Description
#
#       In figure_mode = "lines", and mapping_mode = "ln(z)" this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapLine_Lnz { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse
    #
    #   When this function returns, it should drop down to Unset_LastMouse
    #   Since we must re-use last_x and last_y here.
    #
    set x $mouse_start_xs
    set y $mouse_start_ys
    set px [Pixel_x $x $tp_Mu $tp_Bu]
    set py [Pixel_y $y $tp_Mv $tp_Bv]
    for {set i 0} {$i < $num_segs} {incr i} {

        #
        #   Draw a point in the mapping
        #
        DrawMap_Lnz $px $py $x $y $canv

        #
        #   Update coordinates along the line
        #
        set x [expr $x + $x_seg]
        set y [expr $y + $y_seg]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
    }
}
#   proc -- DrawMapLine_Lnz
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMapLine_Expz
#
#   Description
#
#       In figure_mode = "lines", and mapping_mode = "ln(z)" this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapLine_Expz { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse
    #
    #   When this function returns, it should drop down to Unset_LastMouse
    #   Since we must re-use last_x and last_y here.
    #
    set x $mouse_start_xs
    set y $mouse_start_ys
    set px [Pixel_x $x $tp_Mu $tp_Bu]
    set py [Pixel_y $y $tp_Mv $tp_Bv]
    for {set i 0} {$i < $num_segs} {incr i} {

        #
        #   Draw a point in the mapping
        #
        DrawMap_Expz $px $py $x $y $canv

        #
        #   Update coordinates along the line
        #
        set x [expr $x + $x_seg]
        set y [expr $y + $y_seg]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
    }
}
#   proc -- DrawMapLine_Expz
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------



#   -------------------------------------------------------------------------
#   Procedure -- DrawMapLine_EvalMap
#
#   Description
#
#       (See DrawMapLine_UnitCircle).  Uses the Line Mode but maps with
#       a user-defined map function.
#
#       In figure_mode = "lines", this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapLine_EvalMap { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse
    #
    #   When this function returns, it should drop down to Unset_LastMouse
    #   Since we must re-use last_x and last_y here.
    #
    set x $mouse_start_xs
    set y $mouse_start_ys
    set px [Pixel_x $x $tp_Mu $tp_Bu]
    set py [Pixel_y $y $tp_Mv $tp_Bv]
    for {set i 0} {$i < $num_segs} {incr i} {

        #
        #   Draw a point in the mapping
        #
        DrawMap_EvalMap $px $py $x $y $canv

        #
        #   Update coordinates along the line
        #
        set x [expr $x + $x_seg]
        set y [expr $y + $y_seg]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
    }
}
#   proc -- DrawMapLine_EvalMap
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMapOrtho_UnitCircle
#
#   Description
#
#       In figure_mode = "lines", and mapping_mode = "unit circle" this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#       In this case, it draws a set of lines orthogonal to each other.
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapOrtho_UnitCircle { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv
global max_ortho_lines aspect_ortho_lines

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]
    set angle [expr atan2($dy, $dx)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse


    #   ----------------------------------------------------------
    #   Set up to draw ortho lines
    #   ----------------------------------------------------------
    set ortho_length [expr $line_length * $aspect_ortho_lines]
    set onum_segs [expr round($ortho_length / $line_seg)]
    set dyn [expr $dy * -1]
    set oangle [expr atan2($dx, $dyn)]
    set dx_o [expr $ortho_length * cos($oangle)]
    set dy_o [expr $ortho_length * sin($oangle)]
    set ox_seg [expr $dx_o / $onum_segs]
    set oy_seg [expr $dy_o / $onum_segs]
    set xc [expr $mouse_start_xs + ($dx / 2)]
    set yc [expr $mouse_start_ys + ($dy / 2)]



    #
    #   List of starting points for parallel lines
    #
    set xsp [expr ($dx_o / ($max_ortho_lines + 1))]
    set ysp [expr ($dy_o / ($max_ortho_lines + 1))]
    set msx [expr $mouse_start_xs - ($dx_o / 2)]
    set msy [expr $mouse_start_ys - ($dy_o / 2)]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins [expr $msx + ($xsp * $jj)]
        lappend yBegins [expr $msy + ($ysp * $jj)]
    }

    #
    #   Draw parallel lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x [lindex $xBegins $j]
        set y [lindex $yBegins $j]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $num_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_UnitCircle $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $x_seg]
            set y [expr $y + $y_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    #
    #   List of starting points for perpendicular lines
    #
    set oxsp [expr ($dx / ($max_ortho_lines + 1))]
    set oysp [expr ($dy / ($max_ortho_lines + 1))]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins_o [expr $mouse_start_xs + ($oxsp * $jj)]
        lappend yBegins_o [expr $mouse_start_ys + ($oysp * $jj)]
    }

    #
    #   Draw perpendicular lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x1 [lindex $xBegins_o $j]
        set y1 [lindex $yBegins_o $j]
        set x [expr $x1 - ($dx_o / 2)]
        set y [expr $y1 - ($dy_o / 2)]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $onum_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_UnitCircle $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $ox_seg]
            set y [expr $y + $oy_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    set othro_line_count 1
}
#   proc -- DrawMapOrtho_UnitCircle
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMapOrtho_Lnz
#
#   Description
#
#       In figure_mode = "lines", and mapping_mode = "ln(z)" this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#       In this case, it draws a set of lines orthogonal to each other.
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapOrtho_Lnz { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv
global max_ortho_lines aspect_ortho_lines

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]
    set angle [expr atan2($dy, $dx)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse


    #   ----------------------------------------------------------
    #   Set up to draw ortho lines
    #   ----------------------------------------------------------
    set ortho_length [expr $line_length * $aspect_ortho_lines]
    set onum_segs [expr round($ortho_length / $line_seg)]
    set dyn [expr $dy * -1]
    set oangle [expr atan2($dx, $dyn)]
    set dx_o [expr $ortho_length * cos($oangle)]
    set dy_o [expr $ortho_length * sin($oangle)]
    set ox_seg [expr $dx_o / $onum_segs]
    set oy_seg [expr $dy_o / $onum_segs]
    set xc [expr $mouse_start_xs + ($dx / 2)]
    set yc [expr $mouse_start_ys + ($dy / 2)]



    #
    #   List of starting points for parallel lines
    #
    set xsp [expr ($dx_o / ($max_ortho_lines + 1))]
    set ysp [expr ($dy_o / ($max_ortho_lines + 1))]
    set msx [expr $mouse_start_xs - ($dx_o / 2)]
    set msy [expr $mouse_start_ys - ($dy_o / 2)]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins [expr $msx + ($xsp * $jj)]
        lappend yBegins [expr $msy + ($ysp * $jj)]
    }

    #
    #   Draw parallel lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x [lindex $xBegins $j]
        set y [lindex $yBegins $j]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $num_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_Lnz $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $x_seg]
            set y [expr $y + $y_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    #
    #   List of starting points for perpendicular lines
    #
    set oxsp [expr ($dx / ($max_ortho_lines + 1))]
    set oysp [expr ($dy / ($max_ortho_lines + 1))]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins_o [expr $mouse_start_xs + ($oxsp * $jj)]
        lappend yBegins_o [expr $mouse_start_ys + ($oysp * $jj)]
    }

    #
    #   Draw perpendicular lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x1 [lindex $xBegins_o $j]
        set y1 [lindex $yBegins_o $j]
        set x [expr $x1 - ($dx_o / 2)]
        set y [expr $y1 - ($dy_o / 2)]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $onum_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_Lnz $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $ox_seg]
            set y [expr $y + $oy_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    set othro_line_count 1
}
#   proc -- DrawMapOrtho_Lnz
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- DrawMapOrtho_Expz
#
#   Description
#
#       In figure_mode = "lines", and mapping_mode = "ln(z)" this
#       procedure takes the line endpoints that were created when the
#       mouse was held down and draws the final line (and mapped line).
#
#       In this case, it draws a set of lines orthogonal to each other.
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapOrtho_Expz { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv
global max_ortho_lines aspect_ortho_lines

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]
    set angle [expr atan2($dy, $dx)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse


    #   ----------------------------------------------------------
    #   Set up to draw ortho lines
    #   ----------------------------------------------------------
    set ortho_length [expr $line_length * $aspect_ortho_lines]
    set onum_segs [expr round($ortho_length / $line_seg)]
    set dyn [expr $dy * -1]
    set oangle [expr atan2($dx, $dyn)]
    set dx_o [expr $ortho_length * cos($oangle)]
    set dy_o [expr $ortho_length * sin($oangle)]
    set ox_seg [expr $dx_o / $onum_segs]
    set oy_seg [expr $dy_o / $onum_segs]
    set xc [expr $mouse_start_xs + ($dx / 2)]
    set yc [expr $mouse_start_ys + ($dy / 2)]



    #
    #   List of starting points for parallel lines
    #
    set xsp [expr ($dx_o / ($max_ortho_lines + 1))]
    set ysp [expr ($dy_o / ($max_ortho_lines + 1))]
    set msx [expr $mouse_start_xs - ($dx_o / 2)]
    set msy [expr $mouse_start_ys - ($dy_o / 2)]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins [expr $msx + ($xsp * $jj)]
        lappend yBegins [expr $msy + ($ysp * $jj)]
    }

    #
    #   Draw parallel lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x [lindex $xBegins $j]
        set y [lindex $yBegins $j]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $num_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_Expz $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $x_seg]
            set y [expr $y + $y_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    #
    #   List of starting points for perpendicular lines
    #
    set oxsp [expr ($dx / ($max_ortho_lines + 1))]
    set oysp [expr ($dy / ($max_ortho_lines + 1))]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins_o [expr $mouse_start_xs + ($oxsp * $jj)]
        lappend yBegins_o [expr $mouse_start_ys + ($oysp * $jj)]
    }

    #
    #   Draw perpendicular lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x1 [lindex $xBegins_o $j]
        set y1 [lindex $yBegins_o $j]
        set x [expr $x1 - ($dx_o / 2)]
        set y [expr $y1 - ($dy_o / 2)]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $onum_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_Expz $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $ox_seg]
            set y [expr $y + $oy_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    set othro_line_count 1
}
#   proc -- DrawMapOrtho_Expz
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------



#   -------------------------------------------------------------------------
#   Procedure -- DrawMapOrtho_EvalMap
#
#   Description
#
#       Does the Ortho lines drawing and maps using the current custom map.
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc DrawMapOrtho_EvalMap { canv } {
global mouse_start_xs mouse_start_ys
global mouse_end_xs mouse_end_ys
global Scaling
global tp_Mu tp_Bu tp_Mv tp_Bv
global max_ortho_lines aspect_ortho_lines

    #
    #   Divide the domain into a number of segments
    #   Use "Scaling" as the default divisor
    #
    set dx [expr $mouse_end_xs - $mouse_start_xs]
    set dy [expr $mouse_end_ys - $mouse_start_ys]
    set dx2 [expr $dx * $dx]
    set dy2 [expr $dy * $dy]
    set line_length [expr sqrt($dx2 + $dy2)]
    set angle [expr atan2($dy, $dx)]

    set line_seg $Scaling
    set num_segs [expr round($line_length / $line_seg)]
    set x_seg [expr $dx / $num_segs]
    set y_seg [expr $dy / $num_segs]

    #
    #   Draw the line
    #   Note: we call Unset_LastMouse here because we're done dragging
    #   the mouse, and we will re-use the last_x and last_y variables
    #   in the loop that draws the final line.
    #   After this procedure call we must call Unset_LastMouse again before
    #   the next line is drawn.
    #
    Unset_LastMouse


    #   ----------------------------------------------------------
    #   Set up to draw ortho lines
    #   ----------------------------------------------------------
    set ortho_length [expr $line_length * $aspect_ortho_lines]
    set onum_segs [expr round($ortho_length / $line_seg)]
    set dyn [expr $dy * -1]
    set oangle [expr atan2($dx, $dyn)]
    set dx_o [expr $ortho_length * cos($oangle)]
    set dy_o [expr $ortho_length * sin($oangle)]
    set ox_seg [expr $dx_o / $onum_segs]
    set oy_seg [expr $dy_o / $onum_segs]
    set xc [expr $mouse_start_xs + ($dx / 2)]
    set yc [expr $mouse_start_ys + ($dy / 2)]



    #
    #   List of starting points for parallel lines
    #
    set xsp [expr ($dx_o / ($max_ortho_lines + 1))]
    set ysp [expr ($dy_o / ($max_ortho_lines + 1))]
    set msx [expr $mouse_start_xs - ($dx_o / 2)]
    set msy [expr $mouse_start_ys - ($dy_o / 2)]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins [expr $msx + ($xsp * $jj)]
        lappend yBegins [expr $msy + ($ysp * $jj)]
    }

    #
    #   Draw parallel lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x [lindex $xBegins $j]
        set y [lindex $yBegins $j]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $num_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_EvalMap $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $x_seg]
            set y [expr $y + $y_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    #
    #   List of starting points for perpendicular lines
    #
    set oxsp [expr ($dx / ($max_ortho_lines + 1))]
    set oysp [expr ($dy / ($max_ortho_lines + 1))]
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set jj [expr $j + 1]
        lappend xBegins_o [expr $mouse_start_xs + ($oxsp * $jj)]
        lappend yBegins_o [expr $mouse_start_ys + ($oysp * $jj)]
    }

    #
    #   Draw perpendicular lines
    #
    for {set j 0} {$j < $max_ortho_lines} {incr j} {
        set x1 [lindex $xBegins_o $j]
        set y1 [lindex $yBegins_o $j]
        set x [expr $x1 - ($dx_o / 2)]
        set y [expr $y1 - ($dy_o / 2)]
        set px [Pixel_x $x $tp_Mu $tp_Bu]
        set py [Pixel_y $y $tp_Mv $tp_Bv]
        for {set i 0} {$i < $onum_segs} {incr i} {

            #
            #   Draw a point in the mapping
            #
            DrawMap_EvalMap $px $py $x $y $canv

            #
            #   Update coordinates along the line
            #
            set x [expr $x + $ox_seg]
            set y [expr $y + $oy_seg]
            set px [Pixel_x $x $tp_Mu $tp_Bu]
            set py [Pixel_y $y $tp_Mv $tp_Bv]
        }
        Unset_LastMouse
    }


    set othro_line_count 1
}
#   proc -- DrawMapOrtho_EvalMap
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------




#   -------------------------------------------------------------------------
#   Procedure -- Draw_Point
#
#   Description
#
#       Low-level procedure (above Draw_Mouse_Point) for plotting a point.
#       This procedure is used when no mapping is involved.
#
#   Arguments
#
#       x -- the x location of the mouse (pixel coordinate)
#
#       y -- the y location of the mouse (pixel coordinate)
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_Point { x y canv } {
global last_x last_y
global figures_list
global curr_draw_color

    #
    #	draw the mouse points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_x] != 0 } {
        set id2 [Draw_Mouse_Point $x $y $last_x $last_y $curr_draw_color\
                 $canv]
    }

    #
    #	save each figure in a list
    #	for future manipulation (or clear)
    #
    if { [info exist id2] != 0 } {
        lappend figures_list $id2
    }

    set last_x $x
    set last_y $y
}
#   proc -- Draw_Point
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Draw_MPoint
#
#   Description
#
#       Low-level procedure (above Draw_Mouse_Point) for plotting a point
#       in an "eval" string mapping.
#
#   Arguments
#
#       x -- the x location of the mouse (pixel coordinate)
#
#       y -- the y location of the mouse (pixel coordinate)
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_MPoint { x y canv } {
global last_x last_y
global figures_list
global curr_map_color

    #
    #	draw the mouse points
    #   (first point is not draw--not noticable in "dots", "ovals")
    #
    if { [info exist last_x] != 0 } {
        set id2 [Draw_Mouse_Point $x $y $last_x $last_y $curr_map_color\
                 $canv]
    }

    #
    #	save each figure in a list
    #	for future manipulation (or clear)
    #
    if { [info exist id2] != 0 } {
        lappend figures_list $id2
    }

    set last_x $x
    set last_y $y
}
#   proc -- Draw_MPoint
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Unset_LastMouse
#
#   Description
#
#       Clears global variables : the last x,y point values that are used
#       for drawing line segments between the last point and current point.
#
#   Arguments
#
#       Void.  Operates on global variables.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Unset_LastMouse { } {
global last_x last_y last_px last_py

    if { [info exist last_px] != 0 } {
        unset last_px
        unset last_py
    }
    if { [info exist last_x] != 0 } {
        unset last_x
        unset last_y
    }
}
#   proc -- Unset_LastMouse
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------



#   -------------------------------------------------------------------------
#   Procedure -- Update_Stringy_Line
#
#   Description
#
#       Draw a temporary, rubber band like, line from where the user has
#       anchored the mouse and the current mouse position.
#
#   Arguments
#
#       xs
#
#           The current mouse position in math coordinates.
#
#       ys
#
#           The current mouse position in math coordinates.
#
#       canv
#
#           The canvas widget on which to draw the line.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc Update_Stringy_Line { xs ys canv } {
global mouse_start_xs mouse_start_ys
global stringy_line_list
global tp_Mu tp_Bu tp_Mv tp_Bv

    if { [info exist stringy_line_list] != 0 } {
        #
        #   There is an existing string line (we are in the middle of a
        #   mouse-down state before the final line is set).
        #   We delete the current line so that a total new one can be
        #   drawn.
        #
        set imax [llength $stringy_line_list] 
        for {set i 0} {$i < $imax} {incr i} { 
            set lval [lindex $stringy_line_list $i] 
            .f_client.c1 delete $lval  \
        }
        # clear the list 
        unset stringy_line_list
    }

    #
    #   Draw the new stringy line
    #
    set px0 [Pixel_x $mouse_start_xs $tp_Mu $tp_Bu]
    set py0 [Pixel_y $mouse_start_ys $tp_Mv $tp_Bv]
    set px [Pixel_x $xs $tp_Mu $tp_Bu]
    set py [Pixel_y $ys $tp_Mv $tp_Bv]
    set id [$canv create line $px0 $py0 $px $py\
                -fill "#eeccdd" -arrow both]
    if { [info exist id] != 0 } {
        lappend stringy_line_list $id
    }

}
#   proc -- Update_Stringy_Line
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Clear_Stringy_Line
#
#   Description
#
#       Clears the temporary, rubber band like, line from where the user has
#       anchored the mouse and the current mouse position.
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to draw the line.
#
#   Returns
#
#       void
#
#  --------------------------------------------------------------------------
proc Clear_Stringy_Line { canv } {
global stringy_line_list

    if { [info exist stringy_line_list] != 0 } {
        #
        #   There is an existing string line (we are in the middle of a
        #   mouse-down state before the final line is set).
        #   We delete the current line so that a total new one can be
        #   drawn.
        #
        set imax [llength $stringy_line_list] 
        for {set i 0} {$i < $imax} {incr i} { 
            set lval [lindex $stringy_line_list $i] 
            .f_client.c1 delete $lval  \
        }
        # clear the list 
        unset stringy_line_list
    }
}
#   proc -- Clear_Stringy_Line
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Get_Curr_Map_EvalString
#
#   Description
#
#       Gets the map_eval string corresponding to the current map mode.
#
#   Arguments
#
#       none
#
#   Returns
#
#       Returns the map eval string.
#
#  --------------------------------------------------------------------------
proc Get_Curr_Map_EvalString { } {
global mapping_mode
global mapeval_list

    if { $mapping_mode eq 0 } {
        return ""
    }
    set mindex [expr $mapping_mode - 1]
    set map_str [lindex $mapeval_list $mindex]
    return $map_str
}
#   proc -- Get_Curr_Map_EvalString
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Get_Curr_Map_TypeString
#
#   Description
#
#       Gets the map_type string corresponding to the current map mode.
#
#   Arguments
#
#       none
#
#   Returns
#
#       Returns the map type string.
#
#  --------------------------------------------------------------------------
proc Get_Curr_Map_TypeString { } {
global mapping_mode
global mapeval_list

    if { $mapping_mode eq 0 } {
        return ""
    }
    set mindex [expr $mapping_mode - 1]
    set map_str [lindex $maptype_list $mindex]
    return $map_str
}
#   proc -- Get_Curr_Map_TypeString
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Get_Curr_Map_NameString
#
#   Description
#
#       Gets the map_name string corresponding to the current map mode.
#
#   Arguments
#
#       none
#
#   Returns
#
#       Returns the map name string.
#
#  --------------------------------------------------------------------------
proc Get_Curr_Map_NameString { } {
global mapping_mode
global mapeval_list

    if { $mapping_mode eq 0 } {
        return ""
    }
    set mindex [expr $mapping_mode - 1]
    set map_str [lindex $mapname_list $mindex]
    return $map_str
}
#   proc -- Get_Curr_Map_NameString
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Data_Point_Plotting
#
#   Description
#
#	    General plotting of points (called by Gen_Mouse_Motion)
#
#   Arguments
#
#   Returns
#
#  --------------------------------------------------------------------------
proc Data_Point_Plotting { x_mouse y_mouse x_math y_math canv } {
global figure_mode
global figures_list
global mapping_mode
global Last_Predefined_Map_Mode

    #
    #   Select the kind of Mapping (Mobius, Polynomial, etc)
    #
    if { $mapping_mode == 1 } {

        #
        #   mapping_mode==1, Unit Circle Mobius
        #

        if { $figure_mode == 5 } {
            #
			#  figure_mode==5,  Data
			#
            #
            #   Draw (and map) one point
            #
            DrawMap_UnitCircle $x_mouse $y_mouse $x_math $y_math $canv

            #  end  figure mode==5, Data
		}

        # mapping_mode==1
    } elseif { $mapping_mode == 2 } {

        #
        #   mapping_mode==2, ln(z)
        #

        if { $figure_mode == 5 } {
            #
			#  figure_mode==5,  Data
			#
            #
            #   Draw (and map) one point
            #
            DrawMap_Lnz $x_mouse $y_mouse $x_math $y_math $canv

            #  end  figure mode==5, Data
		}

        # mapping_mode==2
    } elseif { $mapping_mode == 3 } {

        #
        #   mapping_mode==3, exp(z)
        #

        if { $figure_mode == 5 } {
            #
			#  figure_mode==5,  Data
			#
            #
            #   Draw (and map) one point
            #
            DrawMap_Expz $x_mouse $y_mouse $x_math $y_math $canv

            #  end  figure mode==5, Data
		}

        # mapping_mode==1
    } elseif { $mapping_mode > $Last_Predefined_Map_Mode } {
        #
        #   Map using one of the user-defined map functions loaded from a
        #   map file
        #
        if { $figure_mode == 5 } {
            #
			#  figure_mode==5,  Data
			#
            #
            #   Draw (and map) one point
            #
            DrawMap_EvalMap $x_mouse $y_mouse $x_math $y_math $canv

            #  end  figure mode==5, Data
		}

    } else {
        #
        #   No mapping -- just draw the points
        #
        if { $figure_mode == 5 } {
            #
			#  figure_mode==5,  Data
			#
            #
            #   Draw a single mouse point
            #
            Draw_Point $x_mouse $y_mouse $canv
            #  end  figure mode==5, Data
		}
        #   No mapping
    }
}
#   proc -- Data_Point_Plotting
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Gen_Point_Plotting
#
#   Description
#
#	    General plotting of points (called by Gen_Mouse_Motion)
#
#   Arguments
#
#       x_mouse -- the x location of the mouse (pixel coordinate)
#
#       y_mouse -- the y location of the mouse (pixel coordinate)
#
#       x_math -- the x math coordinate
#
#       y_math -- the y math coordinate
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Gen_Point_Plotting { x_mouse y_mouse x_math y_math canv } {
global figure_mode
global B1_down
global figures_list
global mapping_mode
global mouse_start_x mouse_start_y mouse_start_xs mouse_start_ys
global mouse_end_x mouse_end_y mouse_end_xs mouse_end_ys
global last_x last_y last_px last_py
global Last_Predefined_Map_Mode

    #
    #   Select the kind of Mapping (Mobius, Polynomial, etc)
    #
    if { $mapping_mode == 1 } {

        #
        #   mapping_mode==1, Unit Circle Mobius
        #

        if { $figure_mode == 1 } {
            #
			#  figure_mode==1,  "freehand"
			#
            if { $B1_down == 1 } {
                #
                #	mapping unit circle
                #

                #
                #   Draw (and map) one point
                #
                DrawMap_UnitCircle $x_mouse $y_mouse $x_math $y_math $canv

                #   B1_down
            } else {
                #
                #   Mouse released
                #

                #
                #   Only draw, or define figure, when mouse held down
                #
                Unset_LastMouse
                # Mouse release
            }

            #  end  figure mode==1, freehand
		} elseif { $figure_mode == 2 || $figure_mode == 3 } {
            #
            #   Lines for Unit Circle
            #

            if { $B1_down == 1 } {
                #
                #   Mouse pressed -- line figure mode
                #

                if { [info exist last_x] == 0 } {
                    #
                    #   This is the beginning of the mouse press
                    #
                    set mouse_start_x $x_mouse
                    set mouse_start_y $y_mouse
                    set mouse_start_xs $x_math
                    set mouse_start_ys $y_math
                } else {
                    #
                    #   The mouse continues to be held down
                    #
                    Update_Stringy_Line $x_math $y_math $canv
                }
                set last_x $x_mouse
                set last_y $y_mouse

                #   B1_down
            } else {
                #
                #   Mouse Released for line figure mode
                #

                if { [info exist last_x] != 0 } {
                    #
                    #   This is the end of the mouse press
                    #
                    set mouse_end_x $x_mouse
                    set mouse_end_y $y_mouse
                    set mouse_end_xs $x_math
                    set mouse_end_ys $y_math

                    #
                    #   In case there is a time delay...set the mouse cursor
                    #   to a "watch"
                    #
                    set old_cursor [$canv cget -cursor]
                    $canv config -cursor watch
                    #
                    #   Call routine to draw amd map the Line to a Unit Circle
                    #
                    Clear_Stringy_Line $canv
                    if { $figure_mode == 2 } {
                        DrawMapLine_UnitCircle $canv
                    } elseif { $figure_mode == 3 } {
                        DrawMapOrtho_UnitCircle $canv
                    }

                    #
                    #   reset the mouse cursor to its original state
                    #
                    $canv config -cursor $old_cursor

                } else {
                    #
                    #   The mouse is still inactive
                    #
                }

                Unset_LastMouse
                # Mouse release
            }

            # figure_mode = Lines
		}

        # mapping_mode==1
    } elseif { $mapping_mode == 2 } {

        #
        #   mapping_mode==2, ln(z)
        #

        if { $figure_mode == 1 } {
            #
			#  figure_mode==1,  "freehand"
			#
            if { $B1_down == 1 } {
                #
                #	mapping ln(z)
                #

                #
                #   Draw (and map) one point
                #
                DrawMap_Lnz $x_mouse $y_mouse $x_math $y_math $canv

                #   B1_down
            } else {
                #
                #   Mouse released
                #

                #
                #   Only draw, or define figure, when mouse held down
                #
                Unset_LastMouse
                # Mouse release
            }

            #  end  figure mode==1, freehand
		} elseif { $figure_mode == 2 || $figure_mode == 3 } {
            #
            #   Lines for ln(z)
            #

            if { $B1_down == 1 } {
                #
                #   Mouse pressed -- line figure mode
                #

                if { [info exist last_x] == 0 } {
                    #
                    #   This is the beginning of the mouse press
                    #
                    set mouse_start_x $x_mouse
                    set mouse_start_y $y_mouse
                    set mouse_start_xs $x_math
                    set mouse_start_ys $y_math
                } else {
                    #
                    #   The mouse continues to be held down
                    #
                    Update_Stringy_Line $x_math $y_math $canv
                }
                set last_x $x_mouse
                set last_y $y_mouse

                #   B1_down
            } else {
                #
                #   Mouse Released for line figure mode
                #

                if { [info exist last_x] != 0 } {
                    #
                    #   This is the end of the mouse press
                    #
                    set mouse_end_x $x_mouse
                    set mouse_end_y $y_mouse
                    set mouse_end_xs $x_math
                    set mouse_end_ys $y_math

                    #
                    #   In case there is a time delay...set the mouse cursor
                    #   to a "watch"
                    #
                    set old_cursor [$canv cget -cursor]
                    $canv config -cursor watch
                    #
                    #   Call routine to draw amd map the Line to a Unit Circle
                    #
                    Clear_Stringy_Line $canv
                    if { $figure_mode == 2 } {
                        DrawMapLine_Lnz $canv
                    } elseif { $figure_mode == 3 } {
                        DrawMapOrtho_Lnz $canv
                    }

                    #
                    #   reset the mouse cursor to its original state
                    #
                    $canv config -cursor $old_cursor

                } else {
                    #
                    #   The mouse is still inactive
                    #
                }

                Unset_LastMouse
                # Mouse release
            }

            # figure_mode = Lines
		}

        # mapping_mode==2
    } elseif { $mapping_mode == 3 } {

        #
        #   mapping_mode==3, exp(z)
        #

        if { $figure_mode == 1 } {
            #
			#  figure_mode==1,  "freehand"
			#
            if { $B1_down == 1 } {
                #
                #	mapping exp(z)
                #

                #
                #   Draw (and map) one point
                #
                DrawMap_Expz $x_mouse $y_mouse $x_math $y_math $canv

                #   B1_down
            } else {
                #
                #   Mouse released
                #

                #
                #   Only draw, or define figure, when mouse held down
                #
                Unset_LastMouse
                # Mouse release
            }

            #  end  figure mode==1, freehand
		} elseif { $figure_mode == 2 || $figure_mode == 3 } {
            #
            #   Lines for exp(z)
            #

            if { $B1_down == 1 } {
                #
                #   Mouse pressed -- line figure mode
                #

                if { [info exist last_x] == 0 } {
                    #
                    #   This is the beginning of the mouse press
                    #
                    set mouse_start_x $x_mouse
                    set mouse_start_y $y_mouse
                    set mouse_start_xs $x_math
                    set mouse_start_ys $y_math
                } else {
                    #
                    #   The mouse continues to be held down
                    #
                    Update_Stringy_Line $x_math $y_math $canv
                }
                set last_x $x_mouse
                set last_y $y_mouse

                #   B1_down
            } else {
                #
                #   Mouse Released for line figure mode
                #

                if { [info exist last_x] != 0 } {
                    #
                    #   This is the end of the mouse press
                    #
                    set mouse_end_x $x_mouse
                    set mouse_end_y $y_mouse
                    set mouse_end_xs $x_math
                    set mouse_end_ys $y_math

                    #
                    #   In case there is a time delay...set the mouse cursor
                    #   to a "watch"
                    #
                    set old_cursor [$canv cget -cursor]
                    $canv config -cursor watch
                    #
                    #   Call routine to draw amd map the Line to a Unit Circle
                    #
                    Clear_Stringy_Line $canv
                    if { $figure_mode == 2 } {
                        DrawMapLine_Expz $canv
                    } elseif { $figure_mode == 3 } {
                        DrawMapOrtho_Expz $canv
                    }

                    #
                    #   reset the mouse cursor to its original state
                    #
                    $canv config -cursor $old_cursor

                } else {
                    #
                    #   The mouse is still inactive
                    #
                }

                Unset_LastMouse
                # Mouse release
            }

            # figure_mode = Lines
		}

        # mapping_mode==3
    } elseif { $mapping_mode > $Last_Predefined_Map_Mode } {
        #
        #   Map using one of the user-defined map functions loaded from a
        #   map file
        #
        if { $figure_mode == 1 } {
            #
			#  figure_mode==1,  "freehand"
			#
            if { $B1_down == 1 } {
                #
                #	mapping unit circle
                #

                #
                #   Draw (and map) one point
                #
                DrawMap_EvalMap $x_mouse $y_mouse $x_math $y_math $canv

                #   B1_down
            } else {
                #
                #   Mouse released
                #

                #
                #   Only draw, or define figure, when mouse held down
                #
                Unset_LastMouse
                # Mouse release
            }

            #  end  figure mode==1, freehand
		} elseif { $figure_mode == 2 || $figure_mode == 3 } {
            #
            #   Lines for Unit Circle
            #

            if { $B1_down == 1 } {
                #
                #   Mouse pressed -- line figure mode
                #

                if { [info exist last_x] == 0 } {
                    #
                    #   This is the beginning of the mouse press
                    #
                    set mouse_start_x $x_mouse
                    set mouse_start_y $y_mouse
                    set mouse_start_xs $x_math
                    set mouse_start_ys $y_math
                } else {
                    #
                    #   The mouse continues to be held down
                    #
                    Update_Stringy_Line $x_math $y_math $canv
                }
                set last_x $x_mouse
                set last_y $y_mouse

                #   B1_down
            } else {
                #
                #   Mouse Released for line figure mode
                #

                if { [info exist last_x] != 0 } {
                    #
                    #   This is the end of the mouse press
                    #
                    set mouse_end_x $x_mouse
                    set mouse_end_y $y_mouse
                    set mouse_end_xs $x_math
                    set mouse_end_ys $y_math

                    #
                    #   In case there is a time delay...set the mouse cursor
                    #   to a "watch"
                    #
                    set old_cursor [$canv cget -cursor]
                    $canv config -cursor watch
                    #
                    #   Call routine to draw amd map the Line to a Unit Circle
                    #
                    Clear_Stringy_Line $canv
                    if { $figure_mode == 2 } {
                        DrawMapLine_EvalMap $canv
                    } elseif { $figure_mode == 3 } {
                        DrawMapOrtho_EvalMap $canv
                    }

                    #
                    #   reset the mouse cursor to its original state
                    #
                    $canv config -cursor $old_cursor

                } else {
                    #
                    #   The mouse is still inactive
                    #
                }

                Unset_LastMouse
                # Mouse release
            }

            # figure_mode = Lines
		}

    } else {
        #
        #   No mapping -- just draw the points
        #
        if { $figure_mode == 1 } {
            #
			#  figure_mode==1,  "freehand"
			#
            if { $B1_down == 1 } {
                #
                #	draw the mouse points
                #

                #
                #   Draw a single mouse point
                #
                Draw_Point $x_mouse $y_mouse $canv

                # Mouse down
            } else {
                #
                #   Mouse released
                #

                #
                #   Only draw, or define figure, when mouse held down
                #
                Unset_LastMouse

                #   Mouse released
            }
            #  end  figure mode==1, freehand
		} elseif { $figure_mode == 2 } {
            #
            #   Lines
            #

            # Lines
		}
        #   No mapping
    }

}
#   proc -- Gen_Point_Plotting
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Gen_Mouse_Motion
#
#   Description
#
#	    General handler for motion of the mouse	
#
#   Arguments
#
#       mx -- the x location of the mouse (pixel coordinate)
#
#       my -- the y location of the mouse (pixel coordinate)
#
#       canv -- the handle to the canvas on which to draw.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Gen_Mouse_Motion { mx my canv } {
global use_canvas_scrollbars
global coords_watch
global figure_mode
global B1_down
global Scaling
global Ox Oy
global figures_list
global xmax xmin ymax ymin
global x_width y_height
global px_width py_height
global ycenter xcenter
global tp_Mx tp_Bx tp_My tp_By
global mouse_start_x mouse_start_y mouse_start_xs mouse_start_ys
global mouse_end_x mouse_end_y mouse_end_xs mouse_end_ys
global last_x last_y last_px last_py
global Last_Predefined_Map_Mode

    #
    #   If the canvas has scrollbars, then the mouse coordinates will be
    #   given in the view region, not the scrollregion.
    #
    #   Without canvas scrollbars, the view region and the scrollregion
    #   are the same.
    #
    if { $use_canvas_scrollbars } {
        set xvm [lindex [.f_client.c1 xview] 0]
        set yvm [lindex [.f_client.c1 yview] 0]
        set x [expr round($mx + ($xvm * $px_width))]
        set y [expr round($my + ($yvm * $py_height))]
    } else {
        set x $mx
        set y $my
    }

    #
    #   Convert the pixel (mouse) coordinates to math coordinates.
    #
    set xs [Coord_x $x $tp_Mx $tp_Bx]
    set ys [Coord_y $y $tp_My $tp_By]

    #
    #	update the watch window
    #
    if { $coords_watch > 0 } {
        #
        #	if the watcher is turned on
        #
        .f_displaybar.xbox configure -text "x :  $xs "
        .f_displaybar.ybox configure -text "y :  $ys "
    }

    Gen_Point_Plotting $x $y $xs $ys $canv

}
#   proc -- Gen_Mouse_Motion
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------

#   -------------------------------------------------------------------------
#   Procedure -- Draw_Data_Columns
#
#   Description
#
#       Having read a set of data columns from a file, reorders the data
#       into column pairs, walks down each column pair plotting the paired
#       data set.
#
#   Arguments
#
#       canv
#
#           The canvas widget on which to be drawn
#
#   Returns
#
#       Void
#
#  --------------------------------------------------------------------------
proc Draw_Data_Columns { canv } {
global tp_Mu tp_Bu tp_Mv tp_Bv
global Data_Array

    #
    #   Each array element is a line (row) from the data file
    #   If there are more than two columns, then this is multiple data
    #   sets.
    #
    set first_row 1
    set num_rows [array size Data_Array]

    #
    #   Go through each row and "multiplex" the columns out to separate
    #   lists
    #
    for {set i 1} {$i <= $num_rows} {incr i} {

        if { [info exist data_columns] == 1 } {
            unset data_columns
        }
        #
        #   split this row into data fields
        #
        set row_str $Data_Array($i)
        set data_fields [split $row_str]
        set num_fields [llength $data_fields]
        set num_cols 0
        for {set j 0} {$j < $num_fields} {incr j} {

            if { [string length [lindex $data_fields $j]] eq 0 } {
                continue
            }

            incr num_cols
            lappend data_columns [lindex $data_fields $j]
        }

        if { [info exist data_columns] == 0 } {
            break
        }

        #
        #   1st time through
        #
        if { $first_row eq 1 } {

            set number_of_columns $num_cols
            #
            #   Essablish a list under each column
            #
            for {set j 0} {$j < $num_cols} {incr j} {
                #
                #   Append to a list of lists
                #
                lappend Columns_Head [list [lindex $data_columns $j]]
            }

            set first_row 0
        } else {

            for {set j 0} {$j < $number_of_columns} {incr j} {
                #
                #   Retreive the list under this column
                #
                set tmp_list [lindex $Columns_Head $j]
                #
                #   Append one more to that list
                #
                lappend tmp_list [lindex $data_columns $j]
                #
                #   Replace it back into the Columns_Head
                #
                set Columns_Head [lreplace $Columns_Head $j $j $tmp_list]
            }
        }

        #
        #   go on to next row
        #
    }

    if { [info exist Columns_Head] == 0 } {
        Error_Message "Cannot create Columns_Head in Draw_Data_Columns"
        return
    }
    # Info_Message "Columns_Head: $Columns_Head"

    #
    #   Go through the rows again, plotting each pair of columns
    #
    set c1 0
    set c2 1
    while { $c2 < $number_of_columns } {

        #
        #   Get the next pair of column lists
        #
        set col_list_1 [lindex $Columns_Head $c1]
        set col_list_2 [lindex $Columns_Head $c2]

        for {set i 0} {$i < $num_rows} {incr i} {
            #
            #   Get another pair from the column list
            #
            set xs [lindex $col_list_1 $i]
            set ys [lindex $col_list_2 $i]

            #
            #   Error handling -- ignore an empty set
            #
            if { [string length $xs] eq 0 } {
                continue
            }
            if { [string length $ys] eq 0 } {
                continue
            }

            #
            #   Convert math coordinates to mouse points
            #
            set mx [Pixel_x $xs $tp_Mu $tp_Bu]
            set my [Pixel_y $ys $tp_Mv $tp_Bv]

            #
            #   plot the data pair
            #
            Data_Point_Plotting $mx $my $xs $ys $canv
        }
        Unset_LastMouse

        incr c1
        incr c1
        incr c2
        incr c2
    }
}
#   proc -- Draw_Data_Columns
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_X_Tics_Positive
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_X_Tics_Positive { } {
global id_xtics_p

    if { [info exists id_xtics_p] != 0 } {
        set imax [llength $id_xtics_p]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_xtics_p $i] 
            .f_client.c1 delete $lval
        }
        unset id_xtics_p
    }
}
#   proc -- Cleanup_X_Tics_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_X_Labels_Positive
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_X_Labels_Positive { } {
global id_xlabels_p

    if { [info exists id_xlabels_p] != 0 } {
        set imax [llength $id_xlabels_p]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_xlabels_p $i] 
            .f_client.c1 delete $lval
        }
        unset id_xlabels_p
    }
}
#   proc -- Cleanup_X_Labels_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Draw_X_Tics_Positive
#
#   Description
#
#       Tics on positive x axis.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_X_Tics_Positive { } {
global xdiv
global nx_p
global xmax
global Oy
global py_height
global tic_ht
global id_xtics_p
global tp_Mu tp_Bu
global xlabels_mode
global xlabel_xshift xlabel_yshift
global xlabel_above
global id_xlabels_p

    #
    #   Clean up any previous set of positive x tics
    #
    Cleanup_X_Tics_Positive
    #
    #   Clean up any previous set of positive x labels
    #
    Cleanup_X_Labels_Positive

    if { $xmax < 0 } {
        #
        #	there are no positive x tics
        #
        return
    }

    if { $Oy < 0  || $Oy > $py_height } {
        #
        #	the x-axis is offscreen
        #
        return
    }
    if { $nx_p == 0 } {
        return
    }

    #
    #   Calculate the y-positions of each of the tics
    #
    set ytop [expr $Oy - $tic_ht]
    set ybot [expr $Oy + $tic_ht]

    #
    #   Draw the (positive) axis tics
    #
	set i 1
	while { $i < $nx_p } {

        #
        #   Calculate the mathematical location
        #
        set xpos [expr $i * $xdiv]
        #
        #   Convert to the pixel location
        #
        set pxpos [Pixel_x $xpos $tp_Mu $tp_Bu]
        #
        #   save each tic tag so that it can be erased later
        #
        set idtag [.f_client.c1 create line $pxpos $ytop $pxpos $ybot]
        lappend id_xtics_p $idtag

        #
        #   put labels by the tics
        #
        if { $xlabels_mode } {
            #
            #   label
            #
            set pxxpos [expr $pxpos + $xlabel_xshift]
            if { $xlabel_above } {
                set pyypos [expr $ytop - $xlabel_yshift]
            } else {
                set pyypos [expr $ybot - $xlabel_yshift]
            }
            set idtag_label [.f_client.c1 create text $pxxpos $pyypos\
                 -text $xpos]
            lappend id_xlabels_p $idtag_label

        }

        set i [expr $i+1]
    }

    return
}
#   proc -- Draw_X_Tics_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_Y_Tics_Positive
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_Y_Tics_Positive { } {
global id_ytics_p

    if { [info exists id_ytics_p] != 0 } {
        set imax [llength $id_ytics_p]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_ytics_p $i] 
            .f_client.c1 delete $lval
        }
        unset id_ytics_p
    }
}
#   proc -- Cleanup_Y_Tics_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_Y_Labels_Positive
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_Y_Labels_Positive { } {
global id_ylabels_p

    if { [info exists id_ylabels_p] != 0 } {
        set imax [llength $id_ylabels_p]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_ylabels_p $i] 
            .f_client.c1 delete $lval
        }
        unset id_ylabels_p
    }
}
#   proc -- Cleanup_Y_Labels_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Draw_Y_Tics_Positive
#
#   Description
#
#       Tics on the positive y axis.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_Y_Tics_Positive { } {
global ydiv
global ny_p
global ymax
global Ox
global px_width
global tic_wd
global id_ytics_p
global tp_Mv tp_Bv
global ylabels_mode
global ylabel_xshift ylabel_yshift
global ylabel_right
global id_ylabels_p

    #
    #   Clean up any previous set of positive x tics
    #
    Cleanup_Y_Tics_Positive
    #
    #   Clean up any previous set of positive y labels
    #
    Cleanup_Y_Labels_Positive

    if { $ymax < 0 } {
        #
        #	there are no positive y tics
        #
        return
    }

    if { $Ox < 0  || $Ox > $px_width } {
        #
        #	the y-axis is offscreen
        #
        return
    }
    if { $ny_p == 0 } {
        return
    }

    #
    #   Calculate the x-positions of each of the tics
    #
    set xright [expr $Ox + $tic_wd]
    set xleft  [expr $Ox - $tic_wd]

    #
    #   Draw the (positive) axis tics
    #
	set i 1
	while { $i < $ny_p } {

        #
        #   Calculate the mathematical location
        #
        set ypos [expr $i * $ydiv]
        #
        #   Convert to the pixel location
        #
        set pypos [Pixel_y $ypos $tp_Mv $tp_Bv]
        #
        #   save each tic tag so that it can be erased later
        #
        set idtag [.f_client.c1 create line $xleft $pypos $xright $pypos]
        lappend id_ytics_p $idtag

        #
        #   put labels by the tics
        #
        if { $ylabels_mode } {
            #
            #   label
            #
            set pyypos [expr $pypos - $ylabel_yshift]
            if { $ylabel_right } {
                set pxxpos [expr $xright + $ylabel_xshift]
            } else {
                set pxxpos [expr $xleft + $ylabel_xshift]
            }
            set idtag_label [.f_client.c1 create text $pxxpos $pyypos\
                 -text $ypos]
            lappend id_ylabels_p $idtag_label

        }

        set i [expr $i+1]
    }

    return
}
#   proc -- Draw_Y_Tics_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_X_Tics_Negative
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_X_Tics_Negative { } {
global id_xtics_m

    if { [info exists id_xtics_m] != 0 } {
        set imax [llength $id_xtics_m]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_xtics_m $i] 
            .f_client.c1 delete $lval
        }
        unset id_xtics_m
    }
}
#   proc -- Cleanup_X_Tics_Negative
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_X_Labels_Negative
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_X_Labels_Negative { } {
global id_xlabels_m

    if { [info exists id_xlabels_m] != 0 } {
        set imax [llength $id_xlabels_m]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_xlabels_m $i] 
            .f_client.c1 delete $lval
        }
        unset id_xlabels_m
    }
}
#   proc -- Cleanup_X_Labels_Positive
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Draw_X_Tics_Negative
#
#   Description
#
#       Tics on the negative x axis.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_X_Tics_Negative { } {
global xdiv
global nx_m
global xmin
global Oy
global py_height
global tic_ht
global id_xtics_m
global tp_Mu tp_Bu
global xlabels_mode
global xlabel_xshift xlabel_yshift
global xlabel_above
global id_xlabels_m

    #
    #   Clean up any previous set of negative x tics
    #
    Cleanup_X_Tics_Negative
    #
    #   Clean up any previous set of negative x labels
    #
    Cleanup_X_Labels_Negative

    if { $xmin > 0 } {
        #
        #	there are no negative x tics
        #
        return
    }

    if { $Oy < 0  || $Oy > $py_height } {
        #
        #	the x-axis is offscreen
        #
        return
    }
    if { $nx_m == 0 } {
        return
    }

    #
    #   Calculate the y-positions of each of the tics
    #
    set ytop [expr $Oy - $tic_ht]
    set ybot [expr $Oy + $tic_ht]

    #
    #   Draw the (positive) axis tics
    #
	set i 1
    set j -1
	while { $i < $nx_m } {

        #
        #   Calculate the mathematical location
        #
        set xpos [expr $j * $xdiv]
        #
        #   Convert to the pixel location
        #
        set pxpos [Pixel_x $xpos $tp_Mu $tp_Bu]
        #
        #   save each tic tag so that it can be erased later
        #
        set idtag [.f_client.c1 create line $pxpos $ytop $pxpos $ybot]
        lappend id_xtics_m $idtag

        #
        #   put labels by the tics
        #
        if { $xlabels_mode } {
            #
            #   label
            #
            set pxxpos [expr $pxpos + $xlabel_xshift]
            if { $xlabel_above } {
                set pyypos [expr $ytop - $xlabel_yshift]
            } else {
                set pyypos [expr $ybot - $xlabel_yshift]
            }
            set idtag_label [.f_client.c1 create text $pxxpos $pyypos\
                 -text $xpos]
            lappend id_xlabels_m $idtag_label

        }

        set i [expr $i+1]
        set j [expr $j-1]
    }

    return
}
#   proc -- Draw_X_Tics_Negative
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_Y_Tics_Negative
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_Y_Tics_Negative { } {
global id_ytics_m

    if { [info exists id_ytics_m] != 0 } {
        set imax [llength $id_ytics_m]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_ytics_m $i] 
            .f_client.c1 delete $lval
        }
        unset id_ytics_m
    }
}
#   proc -- Cleanup_Y_Tics_Negative
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_Y_Labels_Negative
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_Y_Labels_Negative { } {
global id_ylabels_m

    if { [info exists id_ylabels_m] != 0 } {
        set imax [llength $id_ylabels_m]
	    for {set i 0} {$i < $imax} {incr i} {
            set lval [lindex $id_ylabels_m $i] 
            .f_client.c1 delete $lval
        }
        unset id_ylabels_m
    }
}
#   proc -- Cleanup_Y_Labels_Negative
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Draw_Y_Tics_Negative
#
#   Description
#
#       Tics on the negative y axis.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_Y_Tics_Negative { } {
global ydiv
global ny_m
global ymin
global Ox
global px_width
global tic_wd
global id_ytics_m
global tp_Mv tp_Bv
global ylabels_mode
global ylabel_xshift ylabel_yshift
global ylabel_right
global id_ylabels_m

    #
    #   Clean up any previous set of negative y tics
    #
    Cleanup_Y_Tics_Negative
    #
    #   Clean up any previous set of negative y labels
    #
    Cleanup_Y_Labels_Negative

    if { $ymin > 0 } {
        #
        #	there are no negative x tics
        #
        return
    }

    if { $Ox < 0  || $Ox > $px_width } {
        #
        #	the y-axis is offscreen
        #
        return
    }
    if { $ny_m == 0 } {
        return
    }

    #
    #   Calculate the y-positions of each of the tics
    #
    set xright [expr $Ox + $tic_wd]
    set xleft  [expr $Ox - $tic_wd]

    #
    #   Draw the (positive) axis tics
    #
	set i 1
    set j -1
	while { $i < $ny_m } {

        #
        #   Calculate the mathematical location
        #
        set ypos [expr $j * $ydiv]
        #
        #   Convert to the pixel location
        #
        set pypos [Pixel_y $ypos $tp_Mv $tp_Bv]
        #
        #   save each tic tag so that it can be erased later
        #
        set idtag [.f_client.c1 create line $xleft $pypos $xright $pypos]
        lappend id_ytics_m $idtag

        #
        #   put labels by the tics
        #
        if { $ylabels_mode } {
            #
            #   label
            #
            set pyypos [expr $pypos - $ylabel_yshift]
            if { $ylabel_right } {
                set pxxpos [expr $xright + $ylabel_xshift]
            } else {
                set pxxpos [expr $xleft + $ylabel_xshift]
            }
            set idtag_label [.f_client.c1 create text $pxxpos $pyypos\
                 -text $ypos]
            lappend id_ylabels_m $idtag_label

        }

        set i [expr $i+1]
        set j [expr $j-1]
    }

    return
}
#   proc -- Draw_Y_Tics_Negative
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Cleanup_Axes
#
#   Description
#
#       Clean up.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Cleanup_Axes { } {
global id_xaxis id_yaxis

    Cleanup_X_Tics_Positive
    Cleanup_X_Labels_Positive
    Cleanup_Y_Tics_Positive
    Cleanup_Y_Labels_Positive
    Cleanup_X_Tics_Negative
    Cleanup_X_Labels_Negative
    Cleanup_Y_Tics_Negative
    Cleanup_Y_Labels_Negative
    if { $id_xaxis != -1 } {
        .f_client.c1 delete $id_xaxis
    }
    if { $id_yaxis != -1 } {
        .f_client.c1 delete $id_yaxis
    }
}
#   proc -- Cleanup_Axes
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------

#   -------------------------------------------------------------------------
#   Procedure -- Draw_Axes
#
#   Description
#
#       Draw the axes.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_Axes { } {
global Ox Oy
global px_width py_height
global id_xaxis id_yaxis
global xaxis_mode yaxis_mode
global xtics_mode ytics_mode


    #
    #   Clean up any previous axis
    #
    Cleanup_Axes

    #
    #   Draw the new x axis
    #
    if { $xaxis_mode } {
        #
        #   axis
        #
        if { $Oy > 0} {
            set id_xaxis [.f_client.c1 create line 0 $Oy $px_width $Oy]
        }
		#
		#   Draw tics
		#
		if { $xtics_mode } {
			Draw_X_Tics_Positive
			Draw_X_Tics_Negative
		}
    }

    #
    #   Draw the new y axis
    #
    if { $yaxis_mode } {
        #
        #   axis
        #
        if { $Ox > 0 } {
            set id_yaxis [.f_client.c1 create line $Ox 0 $Ox $py_height]
        }
		#
		#   Draw tics
		#
		if { $ytics_mode } {
			Draw_Y_Tics_Positive
			Draw_Y_Tics_Negative
		}
    }
}
#   proc -- Draw_Axes
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Draw_Unit_Circle
#
#   Description
#
#       Draw unit circle template.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Draw_Unit_Circle { } {
global Scaling
global Ox Oy
global id_unit_circle

    #
    #   Clean up any previous unit circle
    #
    if { $id_unit_circle != -1 } {
        .f_client.c1 delete $id_unit_circle
    }

    #
    #   calculate circle parameters
    #
    set x1 [expr (-1 / $Scaling) + $Ox]
    set y1 [expr $Oy - (1 / $Scaling)]
    set x2 [expr (1 / $Scaling) + $Ox]
    set y2 [expr $Oy + (1 / $Scaling)]

    #
    #   draw the circle
    #
    set id_unit_circle [.f_client.c1 create oval $x1 $y1 $x2 $y2 -outline gray]
}
#   proc -- Draw_Unit_Circle
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Reset_Mapping
#
#   Description
#
#       Reset variables involved with mapping.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Reset_Mapping { } {
global mapping_mode
global id_unit_circle

    #
    #   Reset_Scaling must be called BEFORE this procedure (if needed).
    #

    #
    #   Check map mode state flag
    #
	if { $mapping_mode == 1 } {
		#
		#   inversion in the unit circle
		#
		Draw_Unit_Circle
	} else {
		#
		#   No mapping
		#
		#
		#   Clean up any previous unit circle
		#
		if { $id_unit_circle != -1 } {
			.f_client.c1 delete $id_unit_circle
		}
    }
    #
    #   Redraw the axes
    #
	Draw_Axes

    #
    #   Reset the mode state in the display
    #
	if { [winfo exists .f_displaybar2.mapmode] != 0 } {
		.f_displaybar2.mapmode config -text [Label_Mapmode $mapping_mode]
	}
}
#   proc -- Reset_Mapping
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Reset_FigureMode
#
#   Description
#
#       Reset variables involved with figure_mode.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Reset_FigureMode { } {
global figure_mode

    #
    #   Reset the mode state in the display
    #
	if { [winfo exists .f_displaybar2.figmode] != 0 } {
		.f_displaybar2.figmode config -text [Label_Figmode $figure_mode]
	}
}
#   proc -- Reset_FigureMode
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Reset_View
#
#   Description
#
#       Reset variable involved with view.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Reset_View { } {
    Reset_Scaling
    Reset_Mapping
}
#   proc -- Reset_View
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Move_Figures
#
#   Description
#
#       Move figures that were drawn to a new x,y location.
#
#   Arguments
#
#       xoffset -- x shift in pixels
#
#       yoffset -- y shift in pixels
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Move_Figures { xoffset yoffset } {
global figures_list
global Scaling

    if { [info exist figures_list] == 0 } {
        return
    }

    set xshift [expr $xoffset * (-1)]
    set yshift $yoffset

    #
    #   Convert the mathematical offsets to pixel offsets
    #
    if { $xoffset != 0 } {
        set pxoffset [expr $xshift / $Scaling]
    } else {
        set pxoffset 0
    }
    if { $yoffset != 0 } {
        set pyoffset [expr $yshift / $Scaling]
    } else {
        set pyoffset 0
    }

    set imax [llength $figures_list] 
    for {set i 0} {$i < $imax} {incr i} { 
        set lval [lindex $figures_list $i] 
        .f_client.c1 move $lval $pxoffset $pyoffset
    }
}
#   proc -- Move_Figures
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Scale_Figures_1
#
#   Description
#
#       Part 1 of the scaling of the figures.  The scaling must be done in
#       2 parts: one before the Reset_Scaling procedure and one after.  This
#       is because the mathematical origin moves when the coordinates are
#       rescaled--unless it happens to coincide with the center of the
#       viewport.
#
#   Arguments
#
#       scale -- scale factor.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Scale_Figures_1 { scale } {
global figures_list
global Scaling Scaling_last
global tp_Mu tp_Bu tp_Mv tp_Bv

    if { [info exist figures_list] == 0 } {
        return
    }

    #
    #   Convert the mathematical origin to pixel offsets
    #   This transformation must be done in the OLD coordinates
    #   (i.e., this function is called before the parameters are
    #   rescaled).
    #
    set pxcen [Pixel_x 0 $tp_Mu $tp_Bu]
    set pycen [Pixel_y 0 $tp_Mv $tp_Bv]

    set imax [llength $figures_list] 
    for {set i 0} {$i < $imax} {incr i} { 
        set lval [lindex $figures_list $i] 
        .f_client.c1 scale $lval $pxcen $pycen $scale $scale
    }
}
#   proc -- Scale_Figures_1
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#   -------------------------------------------------------------------------
#   Procedure -- Scale_Figures_2
#
#   Description
#
#       Part 2 of the scaling of the figures.  The scaling must be done in
#       2 parts.
#       This part is called after the Reset_Scaling procedure is called.
#       The first part scales the figures with respect to the old origin,
#       and this part shifts the figures to follow where the origin moved.
#       (The origin moves merely because of the scaling).
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Scale_Figures_2 { } {
global figures_list
global Ox Oy Ox_last Oy_last

    if { [info exist figures_list] == 0 } {
        return
    }

    set Oxdiff [expr $Ox - $Ox_last]
    set Oydiff [expr $Oy - $Oy_last]

    set imax [llength $figures_list] 
    for {set i 0} {$i < $imax} {incr i} { 
        set lval [lindex $figures_list $i] 
        .f_client.c1 move $lval $Oxdiff $Oydiff
    }
}
#   proc -- Scale_Figures_2
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------

#   -------------------------------------------------------------------------
#   Procedure -- Create_Canvas
#
#   Description
#
#       Does the canvas creation.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Create_Canvas { } {
global use_canvas_scrollbars
global wd_Client ht_Client
global px_width py_height
global canvas_magnification
global xsh_cent_cv ysh_cent_cv

    if { $use_canvas_scrollbars } {
        #
        #   TBD: cannot set canvas dimensions, "canvas_magnification" and
        #   "use_canvas_scrollbars" in config file, because canvas is created
        #   before the config file is read.
        #

        #
        #   Set scrollregion of the canvas bigger that the view area because
        #   scrollbar can move to the far corners
        #
        set px_width  [expr $wd_Client * $canvas_magnification]
        set py_height [expr $ht_Client * $canvas_magnification]
        #
        #   offset when centering the canvas
        #
        set xsh_cent_cv [expr (1.0 - (1.0 / $canvas_magnification)) / 2.0]
        set ysh_cent_cv [expr (1.0 - (1.0 / $canvas_magnification)) / 2.0]
    } else {
        #
        #   Set scrollregion of the canvas equivalent to view area
        #
        set px_width  [expr $wd_Client * 1.0]
        set py_height [expr $ht_Client * 1.0]
        #
        #   offset when centering the canvas
        #
        set xsh_cent_cv 0
        set ysh_cent_cv 0
    }

    if { $use_canvas_scrollbars } {
        #
        # Canvas code:	with scrollbars
        #
        # scrollregion: the entire canvas
        # width:  the viewable width
        # height: the viewable height
        #
        # The mouse coordinates are relative to the viewable region, not
        # the scrollregion.
        #
        set scrollcorners [list 0 0 $px_width $py_height]
        canvas .f_client.c1 -width $wd_Client -height $ht_Client -bg white \
            -scrollregion $scrollcorners \
            -xscrollcommand [list .f_client.xscroll set] \
            -yscrollcommand [list .f_client.yscroll set]
        scrollbar .f_client.xscroll -orient horizontal -command [list .f_client.c1 xview]
        scrollbar .f_client.yscroll -orient vertical -command [list .f_client.c1 yview]
        pack .f_client.xscroll -side bottom -fill x
        pack .f_client.yscroll -side right -fill y
        pack .f_client.c1 -fill both -expand true
    } else {
        #
        # Canvas code:	without scrollbars
        #
        canvas .f_client.c1 -width $wd_Client -height $ht_Client -bg white
        pack .f_client.c1
    }
}
#   proc -- Create_Canvas
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------

#  --------------------------------------------------------------------------
#   Procedure -- Set_Filename
#
#   Description
#
#       When a new file name is used this porcedure resets path variables.
#
#   Arguments
#
#       file_name -- the new file name.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Set_Filename { file_name } {
global curr_file_fullpath
global curr_file_name
global curr_file_folder

    set curr_file_name $file_name
    set curr_file_folder [pwd]
    set curr_file_fullpath [file join $curr_file_folder $curr_file_name]
}
#  proc -- Set_Filename
#  ---------------------------------------------------------------------

#   -------------------------------------------------------------------------
#   Procedure -- Center_Canvas
#
#   Description
#
#       Centers the canvas when the Center menu option is selected.
#
#   Arguments
#
#       Void.
#
#   Returns
#
#       Void.
#
#  --------------------------------------------------------------------------
proc Center_Canvas { } {
global xsh_cent_cv ysh_cent_cv

    .f_client.c1 xview moveto $xsh_cent_cv
    .f_client.c1 yview moveto $ysh_cent_cv
}
#   proc -- Center_Canvas
#  --------------------------------------------------------------------------
#  --------------------------------------------------------------------------


#	--------------------------------------------------------------------
#	--------------------------------------------------------------------
#	Start program execution
#
#	--------------------------------------------------------------------
#	--------------------------------------------------------------------
ConfigFile 0 $configfile_name
Create_Canvas

#
#   Calculate the initial scaling parameters
#   Calls:
#       Reset_Scaling
#       Reset_Mapping
#
Reset_View
Reset_FigureMode
Center_Canvas

#
#	set binding to mouse to track the x,y, position
#
bind .f_client.c1 <Any-Motion> { Gen_Mouse_Motion %x %y %W }
bind .f_client.c1 <ButtonPress-1> { set B1_down 1 }
bind .f_client.c1 <ButtonRelease-1> { set B1_down 0 }

#
#   The following standard mapping commands
#   are copies of the hard-coded version, but are needed when the Tools
#   menu is set to "Functions".
#

#
#   Inversion in Unit Circle
#
set mapeval_string_1 "set den \[expr \$x * \$x + \$y * \$y\]\n"
set mapeval_string_1 "$mapeval_string_1 set wx \[expr \$x / \$den\]\n"
set mapeval_string_1 "$mapeval_string_1 set wy \[expr \$y / \$den\]\n"
lappend mapeval_list $mapeval_string_1
lappend maptype_list "Inversion"
lappend mapname_list "Unit Circle"
incr max_mapping_mode
#
set mapeval_string_2 "set rp \[expr hypot\(\$x, \$y\)\]\n"
set mapeval_string_2 "$mapeval_string_2 set theta \[expr atan2\(\$y, \$x\)\]\n"
set mapeval_string_2 "$mapeval_string_2 set mp \[Max \$rp 0.000001\]\n"
set mapeval_string_2 "$mapeval_string_2 set wx \[expr log\(\$mp\)\]\n"
set mapeval_string_2 "$mapeval_string_2 set wy \$theta\n"
lappend mapeval_list $mapeval_string_2
lappend maptype_list "log"
lappend mapname_list "ln(z)"
incr max_mapping_mode
#
set mapeval_string_3 "set rp \[expr exp\(\$x\)\]\n"
set mapeval_string_3 "$mapeval_string_3 set wx \[expr \$rp * cos\(\$y\)\]\n"
set mapeval_string_3 "$mapeval_string_3 set wy \[expr \$rp * sin\(\$y\)\]\n"
lappend mapeval_list $mapeval_string_3
lappend maptype_list "exp"
lappend mapname_list "exp(z)"
incr max_mapping_mode
#

#
#   If there is a valid Map File Name, load the map file
#
set load_return [Load_Map_File $Curr_Map_File $curr_canvas]
if { $load_return <= 0 } {
    #
    #   If the load failed, clear the name
    #
    set Curr_Map_File ""
}


